import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "ContentUris.js" as Uris

/**
 * The library container is used to display "Artist", "Albums" and "Genres" tab.
 * It has a list view whose first item is a segmented control. Since the segmented control
 * is part of the list view, it will scroll with the list view. 
 */
Container {
    id: librarySegmentedContainer
    
    property variant libraryListView: libraryListView;
    property variant selectableLibraryDataModel: selectableDataModel;
    
    property variant optionNames: ["Artists", "Albums", "Genres"] // the order of these names should match
                                                                  // the options of segmented control
    property variant noContentView: noContentView
    
    verticalAlignment: VerticalAlignment.Fill
    
    layout: StackLayout {
        orientation: LayoutOrientation.TopToBottom
    }
	
    attachedObjects: [
        AsyncHeaderDataModel {
            objectName: "Albums"
            id: albumsDataModel
            query: MediaDataQueryDecorator {
                query: LibraryHeaderQueryDecorator {
                    query: ContentUriHeaderDataQuery {
                        uri: Uris.albums(["album_id","album_artist_id","thumb_file","album_artist"])
                        headerUri: Uris.albumGroups()
                        indexEnabled: true
                        // headerUri: "" //if header uri is empty, the model will provide one dummy header for all the data
                        onError: showDataQueryErrorDialog(code, message)
                    }
                }
            }
        },
        AsyncHeaderDataModel {
            objectName: "Artists"
            id: artistsDataModel
            query: MediaDataQueryDecorator {
                query: LibraryHeaderQueryDecorator {
                    query: ContentUriHeaderDataQuery {
                        uri: Uris.artists(["album_artist_id","thumb_file"])
                        headerUri: Uris.artistGroups([])
                        indexEnabled: true
                        // headerUri: "" //if header uri is empty, the model will provide one dummy header for all the data
                        onError: showDataQueryErrorDialog(code, message)
                    }
                }
            }
        },
        AsyncHeaderDataModel {
            objectName: "Genres"
            id: genresDataModel
            query: MediaDataQueryDecorator {
                query: ContentUriHeaderDataQuery {
                    uri: Uris.genres(["genre_id","thumb_file"])
                    headerUri: "" //if header uri is empty, the model will provide one dummy header for all the data
                    indexEnabled: true
                    onError: showDataQueryErrorDialog(code, message)
                }
            }
        },
        PrioritizedDataModelDecorator {
            id: artistsPrioritizedDataModel
            model: artistsDataModel
        },
        PrioritizedDataModelDecorator {
            id: albumsPrioritizedDataModel
            model: albumsDataModel
        },
        PrioritizedDataModelDecorator {
            id: genresPrioritizedDataModel
            model: genresDataModel
        },
        SelectableDataModelDecorator {
            id: selectableDataModel
            model: LibraryDataModelDecorator {
                id: libraryDataModel
                model: artistsPrioritizedDataModel
            }
        },
        ListViewConnection {
            id: libraryListViewConnection
            listView: libraryListView
            key: "header"
            thumbKey: "thumb_file"
            defaultThumb: "asset:///images/common/album_placeholder.png"
        }
    ]
	
    onCreationCompleted: {
        libraryListView.multiselectActiveChanged.connect(activeChangedSlot);
        libraryListView.multiSelectHandler.activeChanged.connect(activeChangedSlot);
        
        artistsDataModel.load();
        albumsDataModel.load();
        genresDataModel.load();
    }
	    
	ScreenTitle {
	    largeText: false
	    showNowPlaying: true
	    title: qsTr("Music Library") + Retranslate.onLanguageChanged
	    horizontalAlignment: HorizontalAlignment.Fill
	}
	
    Container {
        id: artistsAlbumsGenres
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        
        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }
        
        layout: DockLayout {
            id: dockLayout
        }
        
        BackgroundWrapper {
            useAltPaint: libraryListView.isMultiSelect
        }
        
        NoContentView {
            id: noContentView
            title: qsTr("No Music") + Retranslate.onLanguageChanged
            subTitle: (MediaApplication.corporate ? qsTr("You are currently in the work space. Please switch to the personal space to see any personal files.\n\nFiles stored on the SD card are not accessible from the work space. Please switch to the personal space to see the files stored on the SD card.") 
            : qsTr("You can use BlackBerry Link to sync music between your device and computer. \n\nTo download BlackBerry Link on your computer, visit\nwww.blackberry.com/BlackBerryLink")) + Retranslate.onLanguageChanged
            pageDataModel: artistsDataModel
            image: "asset:///images/common/music_no_content_icon.png"
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Center
        }
        
        Container {
            visible: !noContentView.visible                 
            layout: StackLayout {
                orientation: LayoutOrientation.TopToBottom
            }
    
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
    
            LibraryListView {
                id: libraryListView
                dataModel: selectableDataModel
                contentType: MediaLibraryContentType.AlbumArtistType
                
                scrollIndicatorMode: libraryScrolladex.visible ? ScrollIndicatorMode.None : ScrollIndicatorMode.Default 
                
                property int segmentedControlIndex: 0 // used by invocation framework to change the active option
                property bool segmentedControlEnabled: true // used on multi-select
                
                function switchDataModel(selectedIndex) {
                    switch(selectedIndex) {
                        case 0:
                            contentType = MediaLibraryContentType.AlbumArtistType;
                            selectableDataModel.model.model = artistsPrioritizedDataModel;
                            break;
                        case 1:
                            contentType = MediaLibraryContentType.AlbumType;
                            selectableDataModel.model.model = albumsPrioritizedDataModel;
                            break;
                        case 2:
                            contentType = MediaLibraryContentType.GenreType;
                            selectableDataModel.model.model = genresPrioritizedDataModel;
                            break;
                    }
                    libraryListView.scrollToPosition(ScrollPosition.Beginning, ScrollAnimation.None);
                }
                
                listItemComponents: [
                    ListItemComponent {
                       type: "artists"
                       DeletableThumbsListItem {
                           id: artistsListItem
                           labelText: (!ListItemData["album_artist"] ? "" : ListItemData["album_artist"])
                           thumbUri: (!ListItemData["thumb_file"] ? "" : ListItemData["thumb_file"])
                           contentType: MediaLibraryContentType.AlbumArtistType
                           itemSelected: (!ListItemData["isSelected"] ? false : ListItemData["isSelected"])
                           multiSelectionMode: ListItem.view.isMultiSelect                  
                       }
                    },
                    
                    ListItemComponent {
                        type: "albums"
                        DeletableThumbsListItem {
                            id: albumsListItem
                            labelText: (!ListItemData["album"] ? "" : ListItemData["album"])
                            bottomLabelText: (!ListItemData["album"] ? (!ListItemData["album_artist"] ? "" : ListItemData["album_artist"]) : "")
                            bottomLabelVisible:  bottomLabelText != ""
                            thumbUri: (!ListItemData["thumb_file"] ? "" : ListItemData["thumb_file"])
                            contentType: MediaLibraryContentType.AlbumType
                            itemSelected: (!ListItemData["isSelected"] ? false : ListItemData["isSelected"])
                            multiSelectionMode: ListItem.view.isMultiSelect
                        }
                    },
                    
                    ListItemComponent {
                        type: "genres"
                        DeletableThumbsListItem {
                            id: genresListItem
                            labelText: (!ListItemData["genre"] ? "" : ListItemData["genre"])
                            thumbUri: (!ListItemData["thumb_file"] ? "" : ListItemData["thumb_file"])
                            contentType: MediaLibraryContentType.GenreType
                            itemSelected: (!ListItemData["isSelected"] ? false : ListItemData["isSelected"])
                            multiSelectionMode: ListItem.view.isMultiSelect
                            allowSearchAppWorld: false
                        }
                    },
                    
                    ListItemComponent {
                        type: "header_artists"
                        Header {
                            id: artistsListHeader
                            title: (!ListItemData || !ListItemData["header"] ? "" : ListItemData["header"]) 
                        }                
                    },
                    
                    ListItemComponent {
                        type: "header_albums"
                        Header {
                            id: albumsListHeader
                            title: (!ListItemData || !ListItemData["header"] ? "" : ListItemData["header"])
                        }
                    },
                    
                    ListItemComponent {
                        type: "header_genres"
                        content: [
                            Container {/* NO HEADER FOR GENRE */}
                        ]
                    },
                    
                    ListItemComponent {
                        type: "library_segmented_control"
                        
                        Container {
                            id: segmentedControlContainer
                            topPadding: 2
                            leftPadding: 5
                            
                            SegmentedControl {
                                id: segmentedControl
                                objectName: "segmentedControl"
                                
                                enabled: segmentedControlContainer.ListItem.view.segmentedControlEnabled
                                
                                horizontalAlignment: HorizontalAlignment.Center
                                
                                property int segmentedControlIndex: segmentedControlContainer.ListItem.view.segmentedControlIndex                            
                                Option { id: option1; objectName: "option1"; text: qsTr("Artists") + Retranslate.onLanguageChanged; selected: true }
                                Option { id: option2; objectName: "option2"; text: qsTr("Albums") + Retranslate.onLanguageChanged }
                                Option { id: option3; objectName: "option3"; text: qsTr("Genres") + Retranslate.onLanguageChanged }
                                
                                onSelectedIndexChanged: {
                                    segmentedControlContainer.ListItem.view.switchDataModel(selectedIndex);
                                }
                                
                                onSegmentedControlIndexChanged: {
                                    segmentedControl.setSelectedIndex(segmentedControlIndex);
                                }
                            }
                        }
                    }
                ]
                
                function itemType(data, indexPath) {
                    if (indexPath[0] == 0) {
                        return "library_segmented_control";
                    }
                    
                    switch (contentType) {
	                    case MediaLibraryContentType.AlbumArtistType:
	                        return (indexPath.length == 1) ? "header_artists" : "artists";
	                    case MediaLibraryContentType.AlbumType:
	                        return (indexPath.length == 1) ? "header_albums" : "albums";
	                    case MediaLibraryContentType.GenreType:
	                        return (indexPath.length == 1) ? "header_genres" : "genres";
	                    default:
	                        return ""; // shouldn't happen
                    }
                }
            }
            MusicListViewHelper {
                listView: libraryListView
            }
        }

        Scrolladex {
            id: libraryScrolladex
            verticalAlignment: VerticalAlignment.Fill
            listViewConnection: libraryListViewConnection
            toolTipMode: ToolTipMode.TextWithThumb
            visible: (libraryListView.contentType != MediaLibraryContentType.GenreType) // invisible on "Genres" screen
            fadeListWhileActive: true
            animateListWhileActive: false
        }
    }
    
    function pushDetailsScreen(indexPath) {
        var selectedItem = libraryListView.dataModel.data(indexPath);
        
        switch (libraryListView.contentType) {
	        case MediaLibraryContentType.AlbumArtistType:
		        var page = tabbedPane.currentNavPane.safePushOnePage("asset:///core/ArtistDetails.qml");
                commonAnalytics.instance.logPagePush("ArtistDetails", "LibrarySegmentedContainer");
		        page.albumArtistIdStr = selectedItem.album_artist_id;
		        //: album artist is not known for display on top of album art
		        page.titleStr = selectedItem.album_artist;
		        break;
	        case MediaLibraryContentType.AlbumType:
	            var page = tabbedPane.currentNavPane.safePushOnePage("asset:///core/ArtistDetails.qml");
                commonAnalytics.instance.logPagePush("ArtistDetails", "LibrarySegmentedContainer");
		        page.openAtAlbumId = selectedItem.album_id;
		        page.albumArtistIdStr = selectedItem.album_artist_id;
		        //: album artist is not known for display on top of album art
		        page.titleStr = selectedItem.album_artist;
		        break;
	        case MediaLibraryContentType.GenreType:
		        var page = tabbedPane.currentNavPane.safePushOnePage("asset:///core/GenreDetails.qml");
                commonAnalytics.instance.logPagePush("GenreDetails", "LibrarySegmentedContainer");
		        page.genreIdStr = selectedItem.genre_id;
		        //: unnamed genre
		        page.titleStr = selectedItem.genre;  
	            break;
	        default:
	            console.debug("Error: unknown content type");
        }
    }
    
    // Put the segment given by segmentName into focus.
    function selectSegment(segmentName) {
        var index = getSegmentIndex(segmentName);
        if (index >= 0) {
            libraryListView.switchDataModel(index);
            libraryListView.segmentedControlIndex = index;
        }
    }

    // Return the index in the SegmentedControl of the option with the given text.
    // Returns -1 if optionName is invalid.
    function getSegmentIndex(optionName) {
        // Transform the given optionName into the same case as the list of optionNames.
        var name = optionName.charAt(0).toUpperCase() + optionName.slice(1).toLowerCase();
        return optionNames.indexOf(name);
    }
    
    function activeChangedSlot(active) {
        libraryListView.segmentedControlEnabled = !active;
    }
}
