import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import "layoutconstants.js" as LayoutConstants

/**
 * This control contains buttons at the top and a content view
 * that loads different ways to view content to be added to a playlist.
 *
 * @property playlistDataModel  editable playlist datamodel
 */

Container {
    id: addMusicSegmentedContainer
    
    property variant playlistDataModel;

	property int screenWidth: (OrientationSupport.orientation == UIOrientation.Landscape)
	       ? PrimaryDisplay.pixelSize.height:PrimaryDisplay.pixelSize.width
	
    background: ColorUtils.get(0xff121212)
    
    /**
     * This is the filter action item that will be used enable filtering
     */
    property ActionItem filterActionItem
    
    /**
     * Indicates whether the filter field is visible or not
     * This member property should be and remain private to this class
     */
    property bool filterVisible: false
    
    /**
     * Slot function to toggle visiblity of the filter field and the
     * segmented control.
     */
    function toggleFilter() {
        if (addMusicSegmentedContainer.filterVisible) {
            // hide the filter field, show the segmented control
            toggleFilterOff();
        } else {
            // show the filter field, hide the segmented control
            enableVirtualKeyboard(true);
            toggleFilterOn();
        }
    }
    
    function toggleFilterOff() {
        enableVirtualKeyboard(false);
        // the height of these two fields differ from device to device
        // we want to keep both fields 'visible' at all times so the container will take the height of the bigger one
        // instead of changing its visible property, toggle the opacity
        // we also need to toggle the touch propogation mode so the correct field recieves the touch input
        filterField.opacity = 0.0;
        filterField.touchPropagationMode = TouchPropagationMode.None;
        segmentedControl.opacity = 1.0;
        segmentedControl.touchPropagationMode = TouchPropagationMode.Full;
        addMusicSegmentedContainer.filterVisible = false;
        filterField.resetText();
        setActionItemText();
    }
    
    function toggleFilterOn() {
        // the height of these two fields differ from device to device
        // we want to keep both fields 'visible' at all times so the container will take the height of the bigger one
        // instead of changing its visible property, toggle the opacity
        // we also need to toggle the touch propogation mode so the correct field recieves the touch input
        filterField.opacity = 1.0;
        filterField.touchPropagationMode = TouchPropagationMode.Full;
        segmentedControl.opacity = 0.0;
        segmentedControl.touchPropagationMode = TouchPropagationMode.None;
        addMusicSegmentedContainer.filterVisible = true;
        filterField.requestFocus();
        setActionItemText();        
    }
    
    function enableVirtualKeyboard(enabled) {
		filterField.input.flags = enabled ? TextInputFlag.VirtualKeyboard : TextInputFlag.VirtualKeyboardOff;
    }
    
    /**
     * Sets the title of the filter action item as well as
     * the hint text in the filter field.
     */
    function setFilterTitle() {
        switch(segmentedControl.selectedIndex) {
            case 0:
                filterActionItem.setTitle(qsTr("Filter Songs"));
                filterField.hintText = qsTr("Filter Songs");
                break;
            case 1:
                filterActionItem.setTitle(qsTr("Filter Albums"));
                filterField.hintText = qsTr("Filter Albums");
                break;
            case 2:
                filterActionItem.setTitle(qsTr("Filter Artists"));
                filterField.hintText = qsTr("Filter Artists");
                break;
        }
    }

    /**
     * Set the action item and filter text
     */
    function setActionItemText() {
        if (filterVisible) {
            filterActionItem.setTitle(qsTr("Done"));
        } else {
            setFilterTitle();
        }
    }

    /**
     * Add a key listener that brings up the type to filter box.  This is needed so that when SYM key
     * is pressed, the type to filter box appears.
     */
    keyListeners: [
        KeyListener {
            onKeyPressed: {
                // Toggle on filter when the SYM key is pressed.
                if (LayoutConstants.TYPE_TO_SEARCH_ENABLED && !addMusicSegmentedContainer.filterVisible && event.key == 211) {
                    toggleFilterOn();
                }
            }
        }
    ]

    onPlaylistDataModelChanged: {
		if ( playlistDataModel != null && addMusicViewPort != null ) {
			addMusicViewPort.playlistDataModel = playlistDataModel;
		}
	}

	Container {
	    
	    layout: DockLayout {
	    }
        bottomPadding: LayoutConstants.CONTROL_PADDING
        topPadding: LayoutConstants.CONTROL_PADDING
        rightPadding: LayoutConstants.CONTROL_PADDING
        leftPadding: LayoutConstants.CONTROL_PADDING
	       
	    // turn off layout animations so that when we show the filter field, it does not
	    // animate in size
	    implicitLayoutAnimationsEnabled: false

        TextField {
            id: filterField

            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Center

            // Turn off these animations so we get just the opacity fade in/out
            implicitLayoutAnimationsEnabled: false
                
            // This control needs to start off with opacity 0.0 so
            // that the segmented control can be seen and touched
            opacity: 0.0
            
            inputRoute.primaryKeyTarget: true
            input.flags: TextInputFlag.VirtualKeyboardOff

            onTextChanging: {
                addMusicViewPort.filterTextChanging(text)
                if (text == "" && addMusicSegmentedContainer.filterVisible && filterField.input.flags == TextInputFlag.VirtualKeyboardOff) {
                    toggleFilterOff();
                } else if (text != "" && !addMusicSegmentedContainer.filterVisible) {
                    // toggle the filter on if text was typed in the search bar
                    toggleFilterOn();
                }
            }
        }    

	    SegmentedControl {
            id: segmentedControl
            horizontalAlignment: HorizontalAlignment.Center
            implicitLayoutAnimationsEnabled: false
        
            Option { id: option1; objectName: "option1"; text:qsTr("Songs")  + Retranslate.onLanguageChanged; value:"core/AddTracksContainer.qml"; selected: true }
            Option { id: option2; objectName: "option2"; text:qsTr("Albums") + Retranslate.onLanguageChanged; value:"core/AddAlbumsContainer.qml" }
            Option { id: option3; objectName: "option3"; text:qsTr("Artists") + Retranslate.onLanguageChanged; value:"core/AddArtistsContainer.qml" }

            onSelectedIndexChanged: {
                switch(selectedIndex) {
                    case 0: 
                        pages.contentQMLPath = "asset:///core/AddTracksContainer.qml"
                        break;
                    case 1: 
                        pages.contentQMLPath = "asset:///core/AddAlbumsContainer.qml"
                        break;
                    case 2: 
                        pages.contentQMLPath = "asset:///core/AddArtistsContainer.qml"
                        break;                                        
                }
            
                setFilterTitle();
            }
        
            onCreationCompleted: {
                setFilterTitle();
            }
        }
    }
    
    Container { //viewport
    	id: addMusicViewPort
        property variant playlistDataModel;
        property bool newItems: false;

        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }

        preferredWidth: screenWidth
        maxWidth: screenWidth
        minWidth: screenWidth
        
        signal filterTextChanging(string text)
	   	
	   	onPlaylistDataModelChanged: {
	   	    playlistDataModel.newItemsCount.connect( onNewItemsCountChanged );
	   	}
	   	function onNewItemsCountChanged() {
            newItems = !newItems;
        }

		onNewItemsChanged: {
		    pages.at(0).itemsCountModified = !pages.at(0).itemsCountModified;
		}

	    SwitchableContainer { // stacklayout
	        id: pages
            
            verticalAlignment: VerticalAlignment.Fill
            
            contentQMLPath: "asset:///core/AddTracksContainer.qml"
	        layout: StackLayout {
	            orientation: LayoutOrientation.LeftToRight
	        }
	    }
	}
}
