import bb.cascades 1.0
import bb.cascades.multimedia 1.0

/**
 * A list item visual for lists related to adding new tracks, albums, etc. to
 * a playlist
 *
 * @param	title	 		The top row of the track info
 * @param	subTitle		The bottom row of the track info
 * @param 	track			The track number/index
 * @param 	thumb_file		The uri for the image to display
 * @param   showTrackIndexContainer	The flag which shows/hides the container for track number and thumb image
 * @param   showThumbImage	The flag which shows/hides the thumb image for a list
 * @param   showAddedCount  The flag which shows/hides the addedCount
 * @param   subTitleVisible	The flag which shows/hides the subTitle label of track info
 * @param   addedCount      Number of times an item is on the playlist
 * @signal  addTrackToPlaylist Emitted when "add" button is pressed
 * @signal  removeTrackFromPlaylist Emitted when "remove" button is pressed
 * @signal  itemSelected Emitted when the item is pressed but not add or remove.
 */
Container {
    id: addToPlaylistListItem 

    property string title: ""
    property string track: "" 
    property int addedCount: 0

    //"private" do not document exposure
    property bool isTrackAddedAnimationRunning: false

    // the animation duration for all animations in this screen
    property int __animationDuration: 250

    signal addTrackToPlaylist()
    signal removeTrackFromPlaylist()
    signal itemSelected()

    function onAddTrackAnimCompleted() {
        addToPlaylistListItem.isTrackAddedAnimationRunning = false;
    }

    layout: DockLayout {}

    onAddedCountChanged: {
		if ( addedCount == 0) {
			addedCountLabel.text = "";
		} else {
		    addedCountLabel.text = addedCount;
		}
	}

	// background and divider
    Container {
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Center

        layout: DockLayout {
        }
        preferredHeight: 123
        Divider {
            
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Bottom
            
            visible: !(addToPlaylistListItem.ListItem.indexInSection + 1 == addToPlaylistListItem.ListItem.sectionSize)
        }
    }

    // track number, title, sub-title, add and remove track buttons
    Container {
        id: mainContainer

        //store whether touch event was handled. 
        property bool touchHandled: false

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill		
        
	    layout: StackLayout {
	        orientation: LayoutOrientation.LeftToRight
		}

        onTouch: {
            if (event.isUp() && !touchHandled) {
                addToPlaylistListItem.itemSelected();
            }
            touchHandled = false; //reset touch handled
        }

		
		// track number, track title, sub-title
		Container {
		    id: trackInfoContainer
		    
		    leftPadding:10
		    rightPadding:5
		    
		    horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
		    
            layout: StackLayout {
		        orientation: LayoutOrientation.LeftToRight
			}
            layoutProperties: StackLayoutProperties {                
                spaceQuota:100
            }
	        // track number
            Label {
                verticalAlignment: VerticalAlignment.Center
                textStyle.textAlign: TextAlign.Center
                
                layoutProperties: StackLayoutProperties {                    
                    spaceQuota: 12
                }
                id: labelTrack
                text: (track == "" ? qsTr ("0") + Retranslate.onLocaleChanged : track)
                textStyle.color: ColorUtils.getSecondaryTextColor()
                textStyle.base: SystemDefaults.TextStyles.SubtitleText
                rightMargin: 10
            }
            TrackInfoLabel{
		       id: trackInfo
			   leftPadding: 0
		       verticalAlignment: VerticalAlignment.Center

			   layoutProperties: StackLayoutProperties {		       
			       spaceQuota: 88
			   }
			   title: ((addToPlaylistListItem.title == null || addToPlaylistListItem.title == "") ? qsTr("Unknown") + Retranslate.onLanguageChanged : addToPlaylistListItem.title)
		       titleStyle: addToPlaylistListItem.ListItem.view.titleStyle
		   }	    
	    }
    
		// add and remove track icons
		Container {
		    
		    rightPadding: 20
		    
		    horizontalAlignment: HorizontalAlignment.Right
	        verticalAlignment: VerticalAlignment.Center
		    
		    layout: StackLayout {
		        orientation: LayoutOrientation.RightToLeft
			}   
            // add track icon
		    ImageButton {
		        verticalAlignment: VerticalAlignment.Center
                defaultImageSource: "asset:///images/context/ic_add_stand_alone_dark_theme_inactive.png"  
                pressedImageSource: "asset:///images/context/add_pressed.png"
                //not needed disabledImageSource
		        onClicked: {
                   if(!addToPlaylistListItem.isTrackAddedAnimationRunning) { 
                       addToPlaylistListItem.isTrackAddedAnimationRunning = true;
                       addTrackAnim.play();
                       addToPlaylistListItem.addTrackToPlaylist();
                   }
               }
               leftMargin: 15
               rightMargin: 15
		    }
            // remove track icon
            ImageButton {
                verticalAlignment: VerticalAlignment.Center
                defaultImageSource: "asset:///images/context/ic_subtract_stand_alone_dark_theme_inactive.png" 
                pressedImageSource: "asset:///images/context/remove_pressed.png"
                disabledImageSource: "asset:///images/context/black.png" //for some reason when switching to disabled the pressed image is still shown if no disabled is specified.
                onClicked: {
                    addToPlaylistListItem.removeTrackFromPlaylist();
                }
                enabled: (addToPlaylistListItem.addedCount > 0)
                leftMargin: 15
                rightMargin: 15
            }

            //Added Count
		    Label {
		    	id: addedCountLabel
		    	textStyle.color: ColorUtils.get(0xff606060)
		    	
		    	verticalAlignment: VerticalAlignment.Center
		    	
		    	layoutProperties: StackLayoutProperties {		            
		        }
		    }
		}

        animations: [
	        SequentialAnimation{
	            id: addTrackAnim
	            
	    	    ParallelAnimation {
	    	        ScaleTransition{
	    	            target: trackInfoContainer
	    	            toY: 0.5
	    	            toX: 0.5
	    	            duration: __animationDuration
	    	        }
	                FadeTransition {
	                    target: trackInfoContainer
	                    toOpacity: 0.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    ParallelAnimation {
		    	    TranslateTransition {
	                    target: trackInfoContainer
	                    toY: 300
	                    duration: 100
	                    easingCurve: StockCurve.CubicInOut
	                }
	                ScaleTransition{
	    	            target: trackInfoContainer
	    	            toY: 1
	    	            toX: 1
	    	            duration: 100
	    	        }
	    	    }
	    	    
	    	    ParallelAnimation {
	    	        id: addTrack
	                TranslateTransition {
	                    target: trackInfoContainer
	                    toY: 0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	                FadeTransition {
	                    target: trackInfoContainer
	                    toOpacity: 1.0
	                    duration: __animationDuration
	                    easingCurve: StockCurve.CubicInOut
	                }
	    	    }
	    	    
	    	    onEnded: {
	    	        onAddTrackAnimCompleted();
	    	    }
	    	}
    	]
	}
}
