/**
 * ContentUris.js
 * 
 * Research In Motion Limited proprietary and confidential.
 * Copyright (C) Research In Motion Limited, 2012.
 */

.pragma library

///////////////////////////////////////////////////////////////////////////////
// Internal utility functions
///////////////////////////////////////////////////////////////////////////////

/**
 * Return a copy of array a with all duplicate 
 * items removed.
 * @param a An array.
 * @return An array with no duplicates.
 */
function unique(a){
    a.sort();
    for(var i = 1; i < a.length; ){
        if(a[i-1] == a[i]){
            a.splice(i, 1);
        } else {
            i++;
        }
    }
    return a;
}

/**
 * Create a URI from a prefix, suffix, and array of columns.
 * @param uri The URI, excluding column information.
 * @param columns The columns to add to the URI.
 * @returns The URI.
 */
function createUri(uri,columns) {
    columns = unique(columns);  
    var sep = (uri.indexOf("?") == -1 ? "?" : "&");
    return uri + sep + "columns=" + columns.join(',');
}

/**
 * Get the columns required for tracks displayed in the player screen.
 * @returns An array of columns.
 */
function playerColumns() {
    return ["album","album_artist","artist","fid","duration","fullpath","title","album_id"];
}

///////////////////////////////////////////////////////////////////////////////
//URI qualifiers
///////////////////////////////////////////////////////////////////////////////

function filter(uri, where) {
 return uri + "&where=" + where;
}

/**
 * Group alpha uses the first character, the character case is changed to upper, as the gorup criterion.
 * 
 * @param column The column used to group
 * @returns The custom function used to get the first character of the column.
 */
function groupCriterion(column) {
	return "UPPER(SUBSTR(" + column + ",1,1))"
}

function groupAlpha(uri, column) {
  return uri + "&type=groups.char." + column;
}

///////////////////////////////////////////////////////////////////////////////
// Recent URIs
///////////////////////////////////////////////////////////////////////////////

function recentlyAddedAlbums(columns, limit) {
    columns.push("date_added","album");
    return createUri(
        "content://media/shared/audio/albums?orderby=date_added DESC,CASE WHEN album IS NULL THEN 1 ELSE 0 END,album COLLATE UCOL_SORT&limit=" + limit,
        columns);
}

function recentlyPlayedAlbums(columns, limit) {
    columns.push("date_played");
	return createUri(
        "content://media/played/audio/albums?orderby=date_played DESC&limit=" + limit,
        columns);
}

function recentlyPlayedSongs(columns, limit) {
    columns.push("play_count","date_played")
    return createUri(
        "content://media/shared/audio/members?orderby=date_played DESC&where=play_count>=1&limit=" + limit,
        columns);
}

///////////////////////////////////////////////////////////////////////////////
// All Songs URIs
///////////////////////////////////////////////////////////////////////////////

function allSongs(columns) {
    columns.push("title")
	return createUri(
        "content://media/shared/audio/members?orderby=title COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

function allSongsByGroups(columns) {
    columns.push("title", "title_sort")
	return createUri(
        "content://media/shared/audio/members?orderby="
		+ groupCriterion("title_sort") + " COLLATE UCOL_TITLE_SORT ASC, "
        + "title COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

function allSongsGroups(columns) {
    return groupAlpha(allSongsByGroups(columns), "title_sort");
}

function allSongsPlayback() {    
    return allSongs(playerColumns());
}

function allSongsNoContent() {
    return "content://media/audio/members?columns=fid";
}

function allSongsHeader() {
    return allSongs(
        ["SUM(duration) AS duration","COUNT(*) AS count","GROUP_CONCAT(thumb_file) AS four_square","decade","year"])
        + "&fixkeyid";
}

///////////////////////////////////////////////////////////////////////////////
// Album URIs
///////////////////////////////////////////////////////////////////////////////

function album(columns,album_id) {
    return createUri(
            "content://media/shared/audio/albums/" + album_id, 
            columns);
}

function albums(columns) {
    columns.push("album_artist","album", "album_sort");
	return createUri(
		"content://media/shared/audio/albums"
	        + "?orderby=CASE WHEN album IS NULL THEN 1 ELSE 0 END, "
	        +  groupCriterion("album_sort") + " COLLATE UCOL_TITLE_SORT ASC, "
	        + "album COLLATE UCOL_TITLE_SORT ASC, "
	        + "album_artist COLLATE UCOL_TITLE_SORT ASC", 
		 columns);
}

function albumGroups(columns) {
    return groupAlpha(albums(columns),"album_sort");
}

function albumMembers(columns, album_id) {
    columns.push("disc","track");
    return createUri( 
        "content://media/shared/audio/albums/" + album_id + "/members?orderby=disc ASC,track ASC",
        columns);
}

///////////////////////////////////////////////////////////////////////////////
// Artist URIs
///////////////////////////////////////////////////////////////////////////////

function artist(columns,artist_id) {
    return createUri(
        "content://media/shared/audio/artists/" + artist_id + "/albums",
        columns);
}

function artists(columns) {
	// There are two album_artist_id's that have special meanings, i.e.:
	//   - album_artist_id=0 -> means the album artist is "Unknown"
	//   - album_artist_id=1 -> means the album artist is "Various Artists"
	//
	// Those two special album artists are displayed at the end of "Artists" list screen.
	// To achieve this order, we add a prefix in front of artist name.
	//   - For "Unknown", adds 2
	//   - For "Various Artists", add 1
	//   - For all other artists, add 0
    columns.push("album_artist_sort","album_artist_id","album_artist");
    return createUri(
        "content://media/shared/audio/album_artists"
    		+ "?orderby=CASE WHEN album_artist_id=0 OR album_artist_id=1 THEN 2-album_artist_id ELSE 0 END, "
    		+ groupCriterion("album_artist_sort") + " COLLATE UCOL_TITLE_SORT ASC, "
            + "album_artist COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

function artistGroups(columns) {
    return groupAlpha(artists(columns),"album_artist_sort");
}

function artistMembers(columns, album_artist_id) {
    columns.push("title");
    return createUri(
        "content://media/shared/audio/album_artists/" + album_artist_id + "/members?orderby=title COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

function artistMembersByTrack(columns, album_artist_id) {
    columns.push("disc","track");
    return createUri(
        "content://media/shared/audio/album_artists/" + album_artist_id + "/members?orderby=disc,track ASC",
        columns);
}

function artistAlbums(columns, album_artist_id) {
    columns.push("album","year");
    return createUri(
        "content://media/shared/audio/album_artists/" + album_artist_id + "/albums?orderby=year DESC,album",
        columns);
}

function artistAlbumMembersPlayback(album_artist_id) {
    var columns = playerColumns();
    columns.push("disc","track","year","album_artist_id","album_id");
    return createUri("content://media/shared/audio/members?orderby=year DESC,album_id,disc,track ASC" 
        + "&where=album_artist_id=" + album_artist_id,
        columns);
}

///////////////////////////////////////////////////////////////////////////////
// Genre URIs
///////////////////////////////////////////////////////////////////////////////

function genre(columns,genre_id) {
    return createUri("content://media/shared/audio/genres/" + genre_id, columns);
}


function genres(columns) {
    columns.push("genre");
    return createUri(
        "content://media/shared/audio/genres?orderby=CASE WHEN genre IS NULL THEN 1 ELSE 0 END, genre COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

function genreMembers(columns,genre_id) {
    columns.push("title");
    return createUri(
        "content://media/shared/audio/genres/" + genre_id + "/members?orderby=title COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

///////////////////////////////////////////////////////////////////////////////
// Search URIs
///////////////////////////////////////////////////////////////////////////////

function artistSearch() {
    return "content://media/shared/audio/album_artists?columns=album_artist,thumb_file,album_artist_id,key_id&orderby=album_artist ASC";
}

function albumSearch() {
    return "content://media/shared/audio/albums?columns=album,thumb_file,album_id,album_artist,album_artist_id,key_id&orderby=album ASC";
}

function playlistSearch() {
    return "content://media/shared/audio/playlists?columns=playlist_name,plid,four_square,playlist_path,count,duration,key_id&orderby="
        +"playlist_name COLLATE UCOL_TITLE_SORT ASC";
}

function searchTracks() {
    return "content://media/shared/audio/members?columns=fid,album,album_id,title,thumb_file,album_artist,album_artist_id,filename,date_modified,key_id&orderby="
        +"title COLLATE UCOL_TITLE_SORT ASC";
}



///////////////////////////////////////////////////////////////////////////////
// Playlist URIs
///////////////////////////////////////////////////////////////////////////////

function playlistMembers(columns, plid) {
    columns.push("oid");
    return createUri(
        "content://media/shared/audio/playlists/" + plid + "/members?orderby=oid ASC",
        columns);
}

function playlistPlayback(plid) {
    return filter(
        playlistMembers(["album_artist", "album", "title","playlist_name","artist","plid","fullpath","duration","fid","key_id","playlist_entry_text"],plid),
        "fid NOT NULL");
}

function playlist(columns,plid) {
    return createUri(
        "content://media/shared/audio/playlists/" + plid,
        columns);
}

function playlists(columns) {
    columns.push("playlist_name");
    return createUri(
        "content://media/shared/audio/playlists?orderby=playlist_name COLLATE UCOL_TITLE_SORT ASC",
        columns);
}

///////////////////////////////////////////////////////////////////////////////
//Dynamic playlist URIs
///////////////////////////////////////////////////////////////////////////////

function dynamicPlaylistPlayback() {
	var columns = playerColumns();
	columns.push("play_oid","np_oid","filename","date_modified");
    return createUri(
    		"content://media/audio/nowplaying?orderby=play_oid ASC",
        columns);
}

function dynamicPlaylistEdit() {
	return createUri(
    		"content://media/audio/nowplaying?orderby=np_oid ASC",
    		["np_oid","hash","fid","title","artist","duration"]);
}

function saveDynamicPlayList() {
	return createUri(
    		"content://media/audio/nowplaying?orderby=np_oid ASC",
    		["np_oid","fid","fullpath"]);
}

function lastPlayedPlayListForSaving() {
	return createUri(
    		"content://media/audio/lastplayed?orderby=lp_oid ASC",
    		["lp_oid","fid","fullpath"]);
}

///////////////////////////////////////////////////////////////////////////////
// By Year URIs
///////////////////////////////////////////////////////////////////////////////

function decades() {
    return "content://media/shared/audio/members?columns=decade&groupcolumns=TOPN(title,thumb_file,4) AS four_square&type=groups.int.decade&orderby=decade DESC"
}

function years() {
    return "content://media/shared/audio/members?columns=year&groupcolumns=TOPN(title,thumb_file,4) AS four_square&type=groups.int.year&orderby=year DESC"
}

///////////////////////////////////////////////////////////////////////////////
// Invoke URIs
///////////////////////////////////////////////////////////////////////////////

function invokeLookup(columns) {
    columns.push("fid");
    return createUri("content://media/shared/audio/members", columns);
}

function fourSquare() {
    return "content://media/shared/audio/members?columns=four_square";
}
