import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants
import "ContentUris.js" as Uris

/**
 * A control to display the list of songs for a specific genre
 * The list uses CountDurationHeaderWithCover for the list header
 * and ListItemVisual for the list items
 *
 * @param   genreId  The genre id used to retrieve songs for a specific genre
 *
 */

Container {
    property alias genreId: list.genreIdStr
    id: genreList
    property alias listView:list
    property alias selectableDataModel:list.dataModel    
    property bool isEmpty: false;
    
    rightPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
    leftPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING

    layout: DockLayout {
	}
    attachedObjects: [
        AsyncHeaderDataModel {
            objectName: "GenreList"
            id: dataModel
            query: MediaDataQueryDecorator {
	            query: ContentUriHeaderDataQuery {
	                id: dataQuery   
	                onError:  appContext.showDataQueryErrorDialog(code, message)
	            }
	        }
        },
        SelectableDataModelDecorator {
            id: selectableModel
            model: PrioritizedDataModelDecorator {
                model: CurrentSongDecorator {
                    id: curSongDecorator
                    playbackControllerRef: appContext.playbackController
                    dynamicPlaylistManager: appContext.dynamicPlaylistManagerRef
                    model: dataModel
                }
            }
        }
    ]
    
    animations: [
        LoadListTransition {
            target: list
            dataModel: dataModel
        }
    ]
    
    onGenreIdChanged: {
        dataQuery.uri = Uris.genreMembers([ "album_artist", "album", "artist", "fullpath", "duration", "fid", "filename", "date_modified"], genreId);
        dataQuery.headerUri = Uris.genre(["genre","duration","count","thumb_file"],genreId);
        dataModel.load();
    }
    
    function handleItemsChanged () {
        var wasEmpty = genreList.isEmpty;
        genreList.isEmpty = !dataModel.childCount(null);
        
        if ( isEmpty && !wasEmpty ) {
            tabbedPane.currentNavPane.pop();
        }
    }
    
    SelectableMusicListView{
        id:list
        dataModel: selectableModel
        bottomPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING
        topPadding: LayoutConstants.TRACKS_LIST_VIEW_PADDING

        layout: StackListLayout{}

        property int songIndexPath
        property string genreIdStr
        property variant songDecorator: curSongDecorator

        property bool isMultiSelect: false;
        
        onDataModelChanged: {
            genreList.isEmpty = !dataModel.childCount(null);
            dataModel.itemsChanged.connect( genreList.handleItemsChanged );
        }

        //a function to scroll to the currently playing track
        function scrollToCurrentSong(){
            appContext.playbackController.currentIndexChanged.disconnect(scrollToCurrentSong); 
            songIndexPath = musicUtils.getIndexByKey(appContext.playbackController.currentSong.fid, Uris.genreMembers(["fid","key_id"],genreIdStr), "fid=", dataModel,0);
            list.scrollToAsyncModelItem([0,songIndexPath]); //[0,] is added to indicate every song is under the first and only header
        }
        
    	listItemComponents: [
            ListItemComponent {
                type: ""
                SimpleTwoRowListItemVisual {
                    title: ListItemData["title"]
                    subTitle: ListItemData["artist"]
                    track: ListItem.indexInSection + 1
                    duration: ListItemData["duration"]
                    controller: ListItem.view.songDecorator
                    multiSelectionMode: ListItem.view.isMultiSelect
                    itemSelected: ListItemData["isSelected"]
                }
            },
            ListItemComponent {
    			type: "fullItem"
                ListItemVisual {
	                title: ListItemData["title"]
                    track: ListItem.indexInSection + 1
                    duration: ListItemData["duration"]
                    fullpath: ListItemData["fullpath"]
                    subTitle: ListItemData["artist"]
                    showSubTitle: true
                    itemSelected: ListItemData["isSelected"]
                    multiSelectionMode:ListItem.view.isMultiSelect
                    currSong: ListItemData["currSong"]
                    controller: ListItem.view.songDecorator
                }
    		},
            ListItemComponent {
    			type: "header"			
				CountDurationHeaderWithCover {
				    id: genresListHeader
				    duration: ListItemData["duration"]
			        count: ListItemData["count"]
			        thumb_file: ListItemData["thumb_file"]
			        isDeleted: ListItemData["isDeleted"] == null ? false : ListItemData["isDeleted"]
                    onShuffleList:{
                        appContext.playbackController.currentIndexChanged.connect(ListItem.view.scrollToCurrentSong);         
                        dynamicPlaylistManager.startPlaybackByIndex(
                           Uris.genreMembers(Uris.playerColumns(), ListItem.view.genreIdStr),
                           true, //shuffle on 
                           -1);
                        commonAnalytics.instance.logShuffleAll("GenreListHeader");
                    }
                    
	            }
    		}
    	]

    	onTriggered: {
	        if (!dataModel.hasChildren (indexPath)) {
                var selectedItem = list.dataModel.data (indexPath);
                if (selectedItem.fullpath == appContext.playbackController.currentSong.fullpath && appContext.playbackController.currentSong.fullpath != "" && appContext.playbackController.currentSong.fullpath != null) {
                        //selected an already playing track, play/pause toggle
                    if (appContext.playbackController.playbackState == PlaybackController.PlaybackPlaying) {
                        appContext.playbackController.pause ()
                    } else {
                        appContext.playbackController.play ()
                    }
                } else {
                    dynamicPlaylistManager.startPlaybackByIndex(
                            Uris.genreMembers(Uris.playerColumns(), genreId), 
                            appContext.playbackController.shuffleMode == PlaybackController.ShuffleOn, //shuffle works now
                            indexPath[indexPath.length - 1],
                            "0",
                            PlaybackController.PlaybackPlaying,          
                            selectedItem);
                }
                clearSelection();
                list.select (indexPath, false);
            }
        }
    }
}
