import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0

/**
 * Adds some common functionality and formatting to MusicListView used in Artists/Albums/Genres screens
 */
MusicListView {
    id: list
    
    /**
     * Emitted when multiselect state on this list has changed
     */
    signal multiselectActiveChanged(bool active)
    
    property alias multiHandler: libraryMultiSelectHandler
    property bool isMultiSelect: false
    property variant contentType
    
    layout: GridListLayout {
        headerMode: ListHeaderMode.Sticky
    }
    attachedObjects: [
        MultiSelectHandler {
            id: libraryMultiSelectHandler
            actions: [
                ActionItem {
                    title: qsTr("Play Next")  + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/common/ic_add_to_play_next.png"
                    enabled: dataModel.selectionList.length > 0
                    onTriggered: {
                        appContext.addToNowPlaying(list.contentType, dataModel.selectionList, null, false);
                        commonAnalytics.instance.logMultiSelectAction("PlayNext", dataModel.selectionList.length);
                    }
                },
                ActionItem {
                    title: qsTr("Play Last") + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/common/ic_add_to_play_last.png"
                    enabled: dataModel.selectionList.length > 0 && playerStub.playbackActive
                    onTriggered: {
                        appContext.addToNowPlaying(list.contentType, dataModel.selectionList, null, true);
                        commonAnalytics.instance.logMultiSelectAction("PlayLast", dataModel.selectionList.length);
                    }
                },
                ActionItem {
                    property variant selectionList
                    title: qsTr("Add to Playlist")  + Retranslate.onLanguageChanged
                    imageSource: "asset:///images/playlists/ic_add_to_playlist.png" //USED BY STABILITY TESTING
                    enabled: dataModel.selectionList.length > 0
                    onTriggered: {
                        //open add to playlist screen
                        var page = tabbedPane.currentNavPane.safePushOnePage ("core/SelectPlaylist.qml");
                        commonAnalytics.instance.logPagePush("SelectPlaylist", "LibraryListView");
                        page.contentType = list.contentType;
                        page.idsToAdd = dataModel.selectionList;
                        selectionList = dataModel.selectionList;
                        page.selectedPlaylist.connect(onSelectedPlaylist);
                    }
                    function onSelectedPlaylist(plid) {
                        commonAnalytics.instance.logMultiSelectAction("AddToPlaylist", selectionList.length);
                    }
                },
                DeleteActionItem {
                    property SystemDialog deleteDialog
                    property variant selectionList
                    enabled: dataModel.selectionList.length > 0
                    onTriggered: {
                        selectionList = dataModel.selectionList;
                        deleteDialog = appContext.getDeleteSystemDialog(list.contentType, selectionList.length);
                        deleteDialog.finished.connect(dialogCallback);
                        deleteDialog.show();
                    }
                    function dialogCallback(result) {
                        if (deleteDialog.result == SystemUiResult.ConfirmButtonSelection) {
                            if (selectionList) {
                                deleteManager.deleteContentByIds(list.contentType, selectionList);
                                commonAnalytics.instance.logMultiSelectAction("Delete", selectionList.length);
                            }
                        }
                    }
                }
            ]
            
            onActiveChanged: {
                if (active) {
                    appContext.deactivateMultiSelect.connect(disableMultiSelect);
                    list.isMultiSelect = true;
                    list.rightPadding = 120;
                } else {
                    appContext.deactivateMultiSelect.disconnect(disableMultiSelect);
                    dataModel.clearSelections();
                    list.isMultiSelect = false;
                    list.rightPadding = 0;
                }
                
                
	            // emit our signal that the state changed
	            list.multiselectActiveChanged(active);
            }
            status: qsTr ("%Ln Item(s) Selected", "", dataModel.selectionList.length) + Retranslate.onLocaleOrLanguageChanged
        }
    ]
    onTriggered: {
        if (!dataModel.hasChildren(indexPath)) {
            pushDetailsScreen(indexPath);
        }
    }
    onSelectionChanged: {
        if (isMultiSelect) {
            dataModel.changeSelection(indexPath, !dataModel.isSelected(indexPath));
        } else {
            dataModel.changeSelection(indexPath, selected);
        }
    }
    property MultiSelectActionItem multiSelectActionItem: MultiSelectActionItem {
        multiSelectHandler: libraryMultiSelectHandler
        onTriggered: {
            libraryMultiSelectHandler.active = true;
        }
    }
    
 	property ActionItem playNextActionItem: ActionItem {
        objectName: "LibraryListViewPlayNext"
		title: qsTr("Play Next") + Retranslate.onLanguageChanged
		imageSource: "asset:///images/common/ic_add_to_play_next.png"
		onTriggered: {
			var selectedItem = dataModel.data(selectedIndexPath);
			addToNowPlaying(selectedItem, list.contentType, false);
		}
	}

 	property ActionItem playLastActionItem: ActionItem {
        objectName: "LibraryListViewPlayLast"
		title: qsTr("Play Last") + Retranslate.onLanguageChanged
		imageSource: "asset:///images/common/ic_add_to_play_last.png"
		onTriggered: {
			var selectedItem = dataModel.data(selectedIndexPath);
			addToNowPlaying(selectedItem, list.contentType, true);
		}
	}

    property ActionItem addToPlaylistActionItem: ActionItem {
        objectName: "LibraryListViewAddToPlayList"
        title: qsTr("Add to Playlist")  + Retranslate.onLanguageChanged
        imageSource: "asset:///images/playlists/ic_add_to_playlist.png" //USED BY STABILITY TESTING
        onTriggered: {
            //open add to playlist screen
            var selectedItem = dataModel.data(selectedIndexPath);
            var page = tabbedPane.currentNavPane.safePushOnePage ("core/SelectPlaylist.qml");
            commonAnalytics.instance.logPagePush("SelectPlaylist", "LibraryListView");
            page.contentType = list.contentType;
            switch (list.contentType) {
             case  MediaLibraryContentType.AlbumType:
                 page.idsToAdd = [selectedItem.album_id];
                 break;
             case MediaLibraryContentType.AlbumArtistType:
                 page.idsToAdd = [selectedItem.album_artist_id];
                 break;
             case MediaLibraryContentType.GenreType:
                 page.idsToAdd = [selectedItem.genre_id];
                 break;            
             case MediaLibraryContentType.PlaylistType:
                 page.idsToAdd = [selectedItem.plid];
                 break;       
             default:
                 console.log("have unknown content type");
                 break;
             } 
         }
    }
    
    // function to add the select item to the Now Playing list
    function addToNowPlaying(selectedItem, contentType, last) {

		switch (contentType) {
		case  MediaLibraryContentType.AlbumType:
            appContext.addToNowPlaying(contentType, [selectedItem.album_id], null, last);
			break;
		case MediaLibraryContentType.AlbumArtistType:
            appContext.addToNowPlaying(contentType, [selectedItem.album_artist_id], null, last);
			break;
		case MediaLibraryContentType.GenreType:
            appContext.addToNowPlaying(contentType, [selectedItem.genre_id], null, last);
			break;            
		case MediaLibraryContentType.PlaylistType:
            appContext.addToNowPlaying(contentType, [selectedItem.plid], null, last);
			break;       
		default:
			console.log("have unknown content type");
        }   	    
    }
    
    function disableMultiSelect(){
    	libraryMultiSelectHandler.active = false;
    }
}
