import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.cascades.music 1.0
import bb.system 1.0
import "layoutconstants.js" as LayoutConstants
import "PlUtils.js" as PlUtils

/**
 * This is a page that manages an editable playlist. When playlist is modified and successfully
 * saved, signal playlistSaved is emitted and the owner is responsible to pop the page.
 *
 * If openPlaylistDetailOnSaved is set to true, this page is automatically pop and open
 * the corresponding PlaylistDetail page. playlistSaved signal is not emitted in this case.
 *
 * @param   titleStr            The text to use as the title for this page.
 * @parm    playlistDataModel   The PlaylistDataModel to be edited.
 */

MediaPage {
    id: playlistEditPage
    isDirty: true
    
    property alias titleStr: textField.text
    property alias textField: textField
    // If true, the page is automatically popped and opens PlaylistDetails screen. playlistSaved signal is not
    // emitted in this case.
    property bool openPlaylistDetailOnSaved: false
    property variant playlistDataModel // Dont use alias for custom type or it will result in segfault
    property int errSavingPlaylist:PlaylistEditErrorType.PLERROR_OK
    property bool savedPeekEnabled: true
    property bool loadDataModel: false
    
    // Internal properties
    property SystemDialog confirmDialog
    property SystemDialog errorDialog
    
    signal playlistSaved(string plid, string filename, variant four_square_list)
    
    paneProperties: NavigationPaneProperties {
        backButton: ActionItem {
            id: playlistEditBackButton
            enabled: true 
            objectName: "playlistEditBackButton"
            onTriggered: {
            	backButtonPressed();
           }
        }
    }

   
    property ActionItem done: ActionItem {
                    id: playlistEditDoneActionItem
                    title: qsTr("Save") + Retranslate.onLanguageChanged //USED BY STABILITY TESTING
                    enabled: true 
                    imageSource: "asset:///images/common/ic_done.png"
                    onTriggered: {
                        saveAndStopEditing();
                    }
                }

    property ActionItem addTracks: ActionItem {
                    id: playlistDetailsAddTracksActionItem
                    title: qsTr("Add Songs") + Retranslate.onLanguageChanged
                    enabled: true 
                    imageSource: "asset:///images/playlists/ic_add_songs.png"
                    onTriggered: {
                    
                        //show the add tracks screen
                        var page;
                        page = tabbedPane.currentNavPane.safePushOnePageOnSheet("core/AddMusic.qml");
                        commonAnalytics.instance.logPagePush("AddMusic", "PlaylistEdit");	 
                        page.playlistDataModel = playlistDataModel;    
                    }
                }

    onCreationCompleted:{
        qmlUtils.addActionItemToPage( done, playlistEditPage, 2);
        qmlUtils.addActionItemToPage( addTracks, playlistEditPage, 2);    
        tabbedPane.currentNavPane.backButtonsVisible = true;
        savedPeekEnabled = tabbedPane.currentNavPane.peekEnabled;
        tabbedPane.currentNavPane.peekEnabled = false;
        qmlUtils.enableActionBarDocking(playlistEditPage);
        
        // Defer loading of the datamodel until the screen is fully pushed.
        // This is controlled by parent of this page. It should set the model after
        // the push transition ended. The approach of listening to the transition end
        // signal within this page failed due to the model instance may not be available
        // at the time when the transition ends
        
        // For large playlists, it can take some time to load the full list.
        // For better UX feedback, we want to show a spinner while loading.
        // It needs to be started prior to the dataModel.load() so the spinner
        // has CPU cycles to show itself, which is why its done here.
        activityIndicator.start();
    }
    
    onPlaylistDataModelChanged: {

        playlistDataModel.persistCompleted.connect(onPersistCompleted);

        if (loadDataModel) {
            playlistDataModel.load();
        }
        // We need to load the dataModel before assigning it to the drag-drop list.
        listView.playlistDataModel = playlistDataModel;

        // stop and hide the busy spinner.
        activityIndicator.stop();
    }

    function onPersistCompleted(result, plid, filename, four_square_list) {
        if (result !=PlaylistEditErrorType.PLERROR_OK) {
            handleError(result);
            return;
        }

		analytics.logPlaylistCreation(filename, playlistDataModel.childCount([]));

		if (openPlaylistDetailOnSaved) {
			popPlaylistEditPage();
			var page = tabbedPane.currentNavPane.safePushOnePage("core/PlaylistDetails.qml");
            commonAnalytics.instance.logPagePush("PlaylistDetails", "PlaylistEdit");
			page.playlistIdStr = plid;
			page.titleStr = filename;
			page.thumb1Uri = four_square_list[0];
			page.thumb2Uri = four_square_list[1];
			page.thumb3Uri = four_square_list[2];
			page.thumb4Uri = four_square_list[3];
        } else {
            // need to restore the peek enabled before handing off the pop.
            tabbedPane.currentNavPane.peekEnabled = savedPeekEnabled;
            playlistSaved(plid, filename, four_square_list);
        }
    }
    
    function saveAndStopEditing() {
        var filename = getFileName();

        playlistDataModel.setName(filename);
        if ( playlistDataModel.isModified() ) {
            // The persistData call can consume the CPU and cause the spinner to not show up immediately.
            // Hook up the savePlaylist() to after the spinner is shown to ensure there
            // is no delay in showing the spinner.
            systemDialogUtils.spinWaitToastOpen.connect(onSpinnerShowing);
            //show the spinner
            systemDialogUtils.showSpinWaitToast(qsTr("Saving....") + Retranslate.onLanguageChanged);
        } else {
            popPlaylistEditPage();
        }
    }
    
    function onSpinnerShowing() {
        systemDialogUtils.spinWaitToastOpen.disconnect(onSpinnerShowing);
        playlistDataModel.persistData();
    }
    
    function handleError( error ) {
		errSavingPlaylist = error;
        var errDialog = getErrorDialog();
        errDialog.body = PlUtils.getErrorText(error);
        systemDialogUtils.closeSpinWaitToast();
        errDialog.show();
    }
    
    function backButtonPressed() {
        var filename = getFileName();
        playlistDataModel.setName(filename);
        if (playlistDataModel.isModified()) {
            getConfirmDialog().show();
            return;
        } else {
           popPlaylistEditPage();
        }
    }
    
    function popPlaylistEditPage()
    {
        tabbedPane.currentNavPane.peekEnabled = savedPeekEnabled;
        systemDialogUtils.closeSpinWaitToast();
        tabbedPane.currentNavPane.pop();
    }
    
    function onConfirmDialogSave(){
        analytics.logEvent("PlaylistEditDialogSave");
        saveAndStopEditing();
    }
    
    function onConfirmDialogDiscard() {
        analytics.logEvent("PlaylistEditDialogDiscard");
        popPlaylistEditPage();
    }
    
    function onConfirmDialog(value) {
        //NOTE - avaz: value emitted from signal is not working.  This is a bug with QT.
        //It occurs when the signal and the slot are in different classes. Using the Q_PROPERTY as a workaround
        if(confirmDialog.result == SystemUiResult.CustomButtonSelection) {
            //Save button
            onConfirmDialogSave();
        }else if(confirmDialog.result == SystemUiResult.ConfirmButtonSelection){
            //Discard button
            onConfirmDialogDiscard();
        }
    }
    
    function onErrorDialog(value) {
		if (errSavingPlaylist ==PlaylistEditErrorType.PLERROR_LOWMEMORY) {
            // Pop the current playlist list page and return to the main "Playlists" page 
            // if creating or saving playlist fails due to low internal memory
			console.log("Leaving the current page because Device internal memory is low.");
            popPlaylistEditPage();
        }
    }
     
    function getFileName() {
        return textField.cacheText;
    }

    function setTitle( title ) {
        textField.text = title;
    }

    content: Container {
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        
        Container {
            minHeight: LayoutConstants.SCREEN_TITLE_HEIGHT
            layout: DockLayout {}
            ImageView {
                id: titleBar
                imageSource: "asset:///images/common/core_titlebar_gradient_overlay_d.png"
                verticalAlignment: VerticalAlignment.Fill
                horizontalAlignment: HorizontalAlignment.Fill
            }
            
            Container {
                leftPadding: 16
                rightPadding: 16
                topPadding: 16
		        /* This is the text field where the playlist name is kept
		          cacheText is used as the text property of the text field
		          does not get updated on every keystroke.  In order to get 
		          the visible playlist name we need to update cachetext on 
		          every keystroke.     
		        */
		        TextField {
		            property string cacheText;
		            id: textField
		            hintText: qsTr("Enter Playlist Name") + Retranslate.onLanguageChanged
		            verticalAlignment: VerticalAlignment.Center
		            horizontalAlignment: HorizontalAlignment.Fill
		            textStyle {
		                base: SystemDefaults.TextStyles.TitleText
		            }
		            onTextChanging: {
		                if (text.localeCompare(cacheText) != 0) {
		                    cacheText = text;
		                }
		            }
		        }
	        }
        }

        Container {
			horizontalAlignment: HorizontalAlignment.Fill
			
			layoutProperties: StackLayoutProperties {
			    spaceQuota: 100                
			}
			layout: DockLayout {}

	        Container {
	            id: rootContainer
	            background: Color.Black
	            horizontalAlignment: HorizontalAlignment.Fill
	            verticalAlignment: VerticalAlignment.Fill
	            
	            Container {
	                id: dragDropListContainer
	                layout: DockLayout {}
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Fill
	                DragDropComponent {
	                    id: listView
	                }
	            }
	        } 
	        ActivityIndicator {
	            id: activityIndicator
	            preferredWidth: 256
	            preferredHeight: 256
	            overlapTouchPolicy: OverlapTouchPolicy.Allow
                horizontalAlignment: HorizontalAlignment.Center
                verticalAlignment: VerticalAlignment.Center
            }
	    }
    }
    
    function getConfirmDialog() {
        if (confirmDialog == null) {
            confirmDialog = systemDialogUtils.getSystemDialog();
            confirmDialog.title = qsTr("Unsaved Changes") + Retranslate.onLanguageChanged;
            confirmDialog.body = qsTr("Do you want to save your changes?") + Retranslate.onLanguageChanged;
            confirmDialog.confirmButton.label = qsTr("Discard") + Retranslate.onLanguageChanged;
            confirmDialog.cancelButton.label = qsTr("Cancel") + Retranslate.onLanguageChanged;
            confirmDialog.customButton.label = qsTr("Save") + Retranslate.onLanguageChanged
            confirmDialog.finished.connect(onConfirmDialog);
        }
        return confirmDialog;
    }
    function getErrorDialog() {
        if (errorDialog == null) {
            errorDialog = systemDialogUtils.getSystemDialog();
            errorDialog.title = qsTr("Error saving playlist") + Retranslate.onLanguageChanged;
            errorDialog.confirmButton.label = qsTr("OK") + Retranslate.onLanguageChanged;
            errorDialog.cancelButton.label = undefined;
            errorDialog.finished.connect(onErrorDialog);
        }
        return errorDialog;
    }   
}
