/**
 * AudioRoutingComponent.qml 
 *
 * A component used for displaying a sheet that allows user selection of a connected audio
 * device for audio routing.
 * 
 */

import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import "AudioUtils.js" as AudioUtils

Container {

    property Sheet audioRoutingSheet;
    property variant selectedAudioDevice;
    property int audioManagerHandleValue; // To be set by the app.
    
    // Signal fired when the audio routing sheet closes.
    signal audioRoutingSheetClosed();
    
    // Attached objects
    attachedObjects: [
        AudioManager {
            id: audioManager
        },
        AudioRoutingDataModel {
            id: audioRoutingDataModel
        },
        ComponentDefinition {
            id: audioRoutingSheetDef
            AudioRoutingSheet { }
        }
    ]
    
    /**
     * Gets the icon for the current audio device. 
     **/
    function getSelectedAudioDeviceIcon() {
        return AudioUtils.getAudioDeviceIcon((selectedAudioDevice != null && selectedAudioDevice.connected && !selectedAudioDevice.suspended) ? selectedAudioDevice.deviceType 
            : audioManager.activeAudioDevice.deviceType);
    }
   
    /**
     * Displays the audio routing options sheet. 
     */
    function openAudioRoutingSheet() {
        if (audioRoutingSheet == null) {
            audioRoutingSheet = audioRoutingSheetDef.createObject();
            audioRoutingSheet.audioRoutingDeviceSelected.connect(onAudioRoutingDeviceSelected);
            audioRoutingSheet.closed.connect(audioRoutingSheetClosed);
        }
        audioRoutingSheet.open();
    }
    
    /** 
     * Sets audio routing to the specified device type.  
     */
    function setAudioRouting(deviceType) {
        if (deviceType == AudioDevice.Default) {
            selectedAudioDevice = null;
            audioManager.setAudioRouting(audioManagerHandleValue, deviceType, AudioDevice.Default);
        } else {
            for (var i=0, n=audioRoutingDataModel.childCount([]); i<n; i++) {
                var device = audioRoutingDataModel.data([i]);
                if (device.deviceType == deviceType) {
                    selectedAudioDevice = device;
                    audioManager.setAudioRouting(audioManagerHandleValue, deviceType, AudioDevice.Default);
                    break;
                }
            }
        }
    }
    
    /**
     * Handle's the selection of the audio routing device.
     */
    function onAudioRoutingDeviceSelected(deviceType) {
        setAudioRouting(deviceType);
    }
}

