import bb.cascades 1.0

/**
 * This control allows for four images to be displayed in one thumb list item
 *
 * @param   labelText           The text to use for the top row of the label
 * @param   bottomLabelText     The text to use for the bottom row of the label.  bottomLabelVisible must be set to true for this to appear
 * @param   bottomLabelVisible  This must be set to true for the bottom label to be displayed
 * @param   thumb1Uri           The uri for the top let image
 * @param   thumb2Uri           The uri for the top right image
 * @param   thumb3Uri           The uri for the bottom left image
 * @param   thumb4Uri           The uri for the bottom right image 
 * @param   contentTopMargin    The top margin
 * @param   contentBottomMargin The bottom margin
 * @param   contentRightMargin  The right margin
 * @param   contentLeftMargin   The left margin
 * @param   defaultImage        The uri for the image to use as a default
 * @param   shadowImage         The uri for an image that is overlayed over the album art.  This is typically used to add a shadow border effect
 */

Container {
    id: thumbListItem
    property alias labelText: caption.topText
    property alias topLabelVisible: caption.topVisible
    property alias bottomLabelText: caption.bottomText
    property alias bottomLabelVisible: caption.bottomVisible
    property int   defaultPadding: 0
    property alias contentTopPadding: thumbListItem.topPadding  
    property alias contentBottomPadding: thumbListItem.bottomPadding
    property alias contentRightPadding: thumbListItem.rightPadding
    property alias contentLeftPadding: thumbListItem.leftPadding
    property string thumb1Uri: defaultImage
    property string thumb2Uri: defaultImage
    property string thumb3Uri: defaultImage
    property string thumb4Uri: defaultImage
    property string defaultImage
    property alias overlayImage: overlay.imageSource
    property alias overlayOpacity: overlay.opacity
    property alias pressImageSource: pressImage.imageSource
    property alias selectImageSource: selectImage.imageSource
    property bool itemSelected: false
    property string totalDuration
    
    onThumb1UriChanged: {
        thumb1Image.imageSource = (thumb1Uri == "" || thumb1Uri == null) ? defaultImage : thumb1Uri;
    }
    onThumb2UriChanged: {
        thumb2Image.imageSource = (thumb2Uri == "" || thumb2Uri == null) ? defaultImage : thumb2Uri;
    }
    onThumb3UriChanged: {
        thumb3Image.imageSource = (thumb3Uri == "" || thumb3Uri == null) ? defaultImage : thumb3Uri;
    }
    onThumb4UriChanged: {
        thumb4Image.imageSource = (thumb4Uri == "" || thumb4Uri == null) ? defaultImage : thumb4Uri;
    }

    topPadding: defaultPadding
    bottomPadding: defaultPadding
    rightPadding: defaultPadding
    leftPadding: defaultPadding
    
    horizontalAlignment: HorizontalAlignment.Fill
    verticalAlignment: VerticalAlignment.Fill

    layout: DockLayout {
        id: containerLayout
    }
          
    //Contains a 2x2 grid of images fit to the size of the parent element.
    //This container fills the parent, and contains a stack of stack layouts.
    //The top-level stack layout contains two rows, fit by a "spaceQuota" layout
    //  property that lets it lay out to exactly half the height of the parent
    //  element. The second-level stacks contain two columns, fit by weight
    //  so that the two images are exactly half the width of the parent element.
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        layout: StackLayout {
            orientation: LayoutOrientation.TopToBottom
        }
        //top row of 2
        Container {
            layoutProperties: StackLayoutProperties {
                id: stackLayoutPropertiesSpaceQuotaFifty
                spaceQuota: 50
            }
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }
            // item 1 image - top left
            ImageView {
                id: thumb1Image
                layoutProperties: stackLayoutPropertiesSpaceQuotaFifty
                imageSource: defaultImage
                scalingMethod: ScalingMethod.Fill
                rightMargin: 0
            }
            // item 2 image - top right
            ImageView {
                id: thumb2Image
                layoutProperties: stackLayoutPropertiesSpaceQuotaFifty
                imageSource: defaultImage
                scalingMethod: ScalingMethod.Fill
                leftMargin: 0
            }
        }
        //bottom row of 2
        Container {
            layoutProperties: stackLayoutPropertiesSpaceQuotaFifty
            layout: StackLayout {
                orientation: LayoutOrientation.LeftToRight
            }
            // item 3 image - bottom left
            ImageView {
                id: thumb3Image
                layoutProperties: stackLayoutPropertiesSpaceQuotaFifty
                imageSource: defaultImage
                scalingMethod: ScalingMethod.Fill
                rightMargin: 0
            }
            // item 4 image - bottom right
            ImageView {
                id: thumb4Image
                layoutProperties: stackLayoutPropertiesSpaceQuotaFifty
                imageSource: defaultImage
                scalingMethod: ScalingMethod.Fill
                leftMargin: 0
            }
        }
    }
    
    ThumbsListItemCaption {
        id: caption
    }
    
    //shadow for the image
    ImageView {
        id: overlay
        opacity: 0.0
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
    }
    
    ImageView {
        id: pressImage
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        scalingMethod: ScalingMethod.Fill
        opacity: 0
        attachedObjects: ImplicitAnimationController {
            enabled: false
            propertyName: "opacity"
        }
        animations: [
            FadeTransition {
                id: showPressedBorderAnimation
                toOpacity: 1
                duration: 50
            },
            FadeTransition {
                id: hidePressedBorderAnimation
                delay: 50
                toOpacity: 0
                duration: 150
                easingCurve: StockCurve.QuadraticOut
            }
        ]
    }
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Fill
        topPadding: 4
        leftPadding: 4
        rightPadding: 4
        bottomPadding: 4
        layout: DockLayout {
        }
        ImageView {
            id: selectImage
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
            scalingMethod: ScalingMethod.Fill
            opacity: 0
            animations: [
                FadeTransition {
                    id: showContextActivationAnimation
                    toOpacity: 0.4
                    duration: 350
                    delay: 100
                    easingCurve: StockCurve.Linear
                },
                FadeTransition {
                    id: hideContextActivationAnimation
                    delay: 50
                    toOpacity: 0
                    duration: 150
                    easingCurve: StockCurve.QuadraticOut
                },
                FadeTransition {
                    id: showSelectAnimation
                    toOpacity: 1
                    duration: 50
                }
            ]
        }
    }
    
    function onActivationChanged() {
        // TODO: workaround for activeChanged triggered on and off when multi select mode is activated
        if (!itemSelected) {
            if (active) {
                // Fade in the blue background color if the list item have a ContextMenu
                if (thumbListItem.contextActions && thumbListItem.contextActions.length > 0) {
                    hideContextActivationAnimation.stop();
                    showContextActivationAnimation.play();
                }
                // Show the blue border
                showPressedBorderAnimation.play();
            } else {
                showContextActivationAnimation.stop();
                hideContextActivationAnimation.play();
                showPressedBorderAnimation.stop();
                hidePressedBorderAnimation.play();
            }
        }
    }
    function onItemSelectedChanged() {
        if (itemSelected) {
            showContextActivationAnimation.stop();
            hideContextActivationAnimation.stop();
            showSelectAnimation.play();
            hidePressedBorderAnimation.stop();
            showPressedBorderAnimation.play();
        } else {
            showSelectAnimation.stop();
            showContextActivationAnimation.stop();
            hideContextActivationAnimation.play();
            showPressedBorderAnimation.stop();
            hidePressedBorderAnimation.play();
        }
    }
}
