import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0


/**
 *  This sheet contains functionality to move items to a certain location.
 *  It will display all albums and items in a folder drill down manner and the user
 *  can navigate to a location where he/she wants to move the files to.
 */
Sheet {
    id: moveSheet

    // The contentUri that describes the root albums.
    property string rootAlbumsContentUri: ""

    // The contentUri that describes the sub albums. The correct folder_id will be added automatically
    property string subAlbumContentUri: ""

    // The contentUri that describes the items. The correct folder_id will be added automatically.
    property string subAlbumMembersContentUri: ""

    // Define the list view bottom padding.
    property int listViewPaddingBottom: 0

    // Defines the padding between grid list items.
    property int gridListItemPadding: 0

    // The number of columns for albums.
    property int albumsColumnCount: 1

    // The number of columns for items.
    property int itemColumnCount: 1

    // The MoveManager that performs the underlying move of files.
    property MoveManager moveManager: null

    // A list containing the key ids for the items that are about to be moved. Taken from a SelectableDataModelDecorator preferably.
    property variant selectionList


    // The type of item that is required. Available options are: video_item, picture_item.
    property string itemType: "default"

    property double itemAspectRatio: itemType == video_item ? 1.767 : 1

    // Constant item types
    property string video_item: "video_item"
    property string picture_item: "picture_item"

    signal dismissed()
    property int selectionListCount: 0
    
    property int moveToNewAlbumTextFieldPadding: 0

    onClosed: {
        var page = navigationPane.top;
        //Remove pages without pop transition until the page stack is empty
        while (page != null && navigationPane.remove(page)) {
            page.destroy();
            page = navigationPane.top;
        }
        moveManager.moveSheetClosed()
    }

    onCreationCompleted: {
        moveManager.moveSheetCreated();
        var albumPage = albumsPageDefinition.createObject();
        albumPage.rootAlbumsContentUri = moveSheet.rootAlbumsContentUri;
        albumPage.subAlbumContentUri = moveSheet.subAlbumContentUri;
        albumPage.subAlbumMembersContentUri = moveSheet.subAlbumMembersContentUri;
        albumPage.albumId = "1";
        albumPage.titleBar.acceptAction.enabled = false;
        pushPage(albumPage);
    }

    onSelectionListChanged: {
        //Wait until selection list is set to load a data model.
        selectedItemsModelQuery.uri = moveSheet.getSelectUri();
        selectedItemsModel.model.load();
    }

    onSelectionListCountChanged: {
        if (selectionListCount == 0) {
            closeSheets();
        }
    }

    attachedObjects: [
        ComponentDefinition {
            id: albumsPageDefinition
            AlbumsPage {
                id: albumsPageId 
                listViewPaddingBottom: moveSheet.listViewPaddingBottom
                gridListItemPadding: moveSheet.gridListItemPadding
                albumsColumnCount: moveSheet.albumsColumnCount
                itemColumnCount: moveSheet.itemColumnCount
                itemAspectRatio: moveSheet.itemAspectRatio
                itemType: moveSheet.itemType

                titleBar: TitleBar {
                    //: Title of the move sheet used for moving pictures and videos to another location
                    title: qsTr("Select Location") + Retranslate.onLanguageChanged
                    acceptAction: ActionItem {
                        //: Move the selected items in the currently viewed folder
                        title: qsTr("Move") + Retranslate.onLanguageChanged
                        onTriggered: {
                            var selectedStorage = albumsPageId.selectedStorage;
                            if (!navigationPane.top.isMultiSource) {
                                selectedStorage = getAlbumSource(navigationPane.top.folderKeyId);
                            }
                            handleMoveSelected(selectedStorage, "");
                        }
                    }
                    dismissAction: ActionItem {
                        //: Cancel the ongoing move operation and return to the application
                        title: qsTr("Cancel") + Retranslate.onLanguageChanged
                        onTriggered: {
                            moveSheet.exitMoveSheet();
                        }
                    }
                }
                actions: [
                    ActionItem {
                        title: qsTr("Move to New Album") + Retranslate.onLanguageChanged
                        imageSource: "asset:///imports/bb/cascades/multimedia/internal/images/ic_add_folder.png"
                        ActionBar.placement: ActionBarPlacement.OnBar
                        onTriggered: {
                            handleMoveToNewAlbum();
                        }
                    }
                ]

                function pushSubPage() {
                    var albumPage = albumsPageDefinition.createObject();
                    pushPage(albumPage);
                    return albumPage;
                }
            }
        },
        HeaderTypeDecorator {
            id: selectedItemsModel
            itemType: moveSheet.itemType
            model: SimpleQueryDataModel {
                query: ContentUriDataQuery {
                    id: selectedItemsModelQuery
                }
                onItemsChanged: {
                    moveSheet.selectionListCount = selectedItemsModel.childCount(Array());
                }
            }
        },
        Delegate {
            id: moveToNewAlbumSheetDelegate
            property variant dataModel: selectedItemsModel
            MoveToNewAlbumSheet {
                id: moveToNewAlbumSheet
                selectedItemsModel: moveToNewAlbumSheetDelegate.dataModel
                itemType: moveSheet.itemType
                itemAspectRatio: moveSheet.itemAspectRatio
                itemColumnCount: moveSheet.itemColumnCount
                isMultiSource: navigationPane.top.albumId == "1" ? moveManager.sdCardAvailable : navigationPane.top.isMultiSource
                textFieldPadding: moveToNewAlbumTextFieldPadding;
                onDismissed: {
                    moveToNewAlbumSheet.closed.connect(moveSheet.destroyMoveToNewAlbumSheet);
                    moveToNewAlbumSheet.close();
                }
                onItemsMoved: {
                    closeSheets();
                }

                function moveItems(selectedStorage,newAlbumName) {
                    if (moveManager.isMediaSyncSuspended()) {
                        moveManager.showMediaSyncSuspendedToast();
                        return false;
                    }
                    var currentAlbum = navigationPane.top;
                    if (!isMultiSource) {
                        //If not multi source and root album, assume internal storage.
                        if (currentAlbum.albumId == "1") {
                            selectedStorage = MediaLibrarySourceType.Internal;
                        } else {
                            selectedStorage = getAlbumSource(currentAlbum.folderKeyId);
                        }
                    }
                    moveSheet.handleMoveSelected(selectedStorage,newAlbumName);
                    return true;
                }
            }
        },
        SimpleQueryDataModel {
            id: sourceQueryModel
        },
        DataQueryFactory {
            id: dataQueryFactory
        }
    ]

    function pushPage(page) {
        navigationPane.push(page);
    }

    function handleMoveSelected(selectedStorage, newAlbumName) {
        if (moveManager == null) {
            console.log("MoveSheet - ERROR: MoveManager is NULL");
            return;
        }
        if (moveManager.isMediaSyncSuspended()) {
            moveManager.showMediaSyncSuspendedToast();
            return;
        }
        var currentAlbum = navigationPane.top; 
        var selectedItemsDataModel = selectedItemsModel.model;
        var selectedItemsCount = selectedItemsDataModel.childCount(Array());
        var keyIdList = Array();
        for (var i = 0; i < selectedItemsCount; i++) {
            var indexPath = Array();
            indexPath.push(i);
            var data = selectedItemsDataModel.data(indexPath);
            keyIdList.push(data.key_id);
        }
        console.log("MoveSheet - Moving " + keyIdList.length + " items to " + currentAlbum.folderKeyId + " on source " + selectedStorage);
        
        if(currentAlbum.albumId == "1"){
            if(moveSheet.itemType == moveSheet.video_item){
                currentAlbum.folderKeyId = "os/";
            }else{
                currentAlbum.folderKeyId = "s/";
            }

        }

        moveManager.moveContentByIds(selectedStorage, MediaLibraryContentType.FileType, keyIdList, currentAlbum.folderKeyId, newAlbumName);
        closeSheets();
    }

    function handleMoveToNewAlbum() {
        console.log("move to new album");
        moveToNewAlbumSheetDelegate.active = true;
        moveToNewAlbumSheetDelegate.object.open();
    }

    function getAlbumSource(folderKeyId) {
        var contentUri = "content://media/shared+captured/files/members?columns=source,path_norm&groupcolumns=source&type=groups.string.path_norm&where=path_norm like '" + folderKeyId.replace(/'/g, "''") + "%'";
        var query = dataQueryFactory.createContentUriDataQuery(sourceQueryModel);
        query.uri = contentUri;
        sourceQueryModel.query = query;
        sourceQueryModel.load();
        var result = sourceQueryModel.data([0]);
        if (result) {
            return result["source"] == 0 ? MediaLibrarySourceType.Internal : MediaLibrarySourceType.SDCard;
        }
        return MediaLibrarySourceType.SDCard;
    }

    function closeSheets() {
        if (moveToNewAlbumSheetDelegate.active) {
            moveToNewAlbumSheetDelegate.object.closed.connect(exitMoveSheet);
            moveToNewAlbumSheetDelegate.object.close();
        } else {
            exitMoveSheet();
        }
    }

    function destroyMoveToNewAlbumSheet() {
        moveToNewAlbumSheetDelegate.active = false;
    }

    function exitMoveSheet() {
        destroyMoveToNewAlbumSheet();
        dismissed();
    }

    function getSelectUri() {
        var uri = "content://media/files/members?columns=file_type,thumb_file,key_id,video.duration,video.title&where=key_id IN(";
        var key_ids = "";
        for (var i = 0; i < moveSheet.selectionList.length; i++) {
            var key_id = moveSheet.selectionList[i];
            if (i != 0) {
                key_ids += ",";
            }
            key_ids += key_id;
        }
        uri += key_ids + ")";
        return uri;
    }

    content: NavigationPane {
        id: navigationPane

        onPopTransitionEnded: {
            page.destroy();
            if (navigationPane.count() == 0) {
                moveSheet.dismissed();
            }
        }
    }
}
