import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import bb.device 1.2
import bb.system 1.0

Background {
    signal rendererSelected (variant renderer)
    
    layout: DockLayout{}
        
    horizontalAlignment: HorizontalAlignment.Fill
        
    property bool showHdmi : false
    property bool showMiracast: false
    property bool popping: false
    property bool dialogUp: false
    property SystemDialog wifiDirectDisableDialog: null
    property int currentRendererTypes: 0
    property bool showMiracastGoWarning: false
    property string miracastGoWarning: ""
    property bool showNoMiracastHdmiConnectedWarning: false
    
    attachedObjects: [
	    NetworkStatusManager{
	        id: networkManager
	        
	        onWifiP2pRoleChanged: {
	        	checkWifiP2pRole();
	        }
	        
	        onWifiDirectEnabledChanged: {
	           console.log("onWifiDirectEnabled");
	           showWiFiDirectDisableDialog();
	        }
	    },
	    SecurityStatus {
	        id: securityStatus
	        
	        onHdmiPlayOnEnabledChanged: {
	            checkForDisabled();
	        }
	        onDlnaPlayOnEnabledChanged: {
	            checkForDisabled();
	        }
	        onMiracastPlayOnEnabledChanged: {
	            checkForDisabled();
	            if(enabled) {
                    // Check if we need to show the Wi-FiDirect disable dialog
                    showWiFiDirectDisableDialog();
	                //If Miracast just  got enabled and HDMI is plugged in then
	                //We need to warn that HDMI prevents Miracast from being used.
                    checkHDMIPreventsMiracast();
	            }
	        }
	    },
	    SystemDialogUtils{
	        id: dialogUtils
	    }
    ]
    
    onCreationCompleted: {
        listView.rendererSelected.connect(rendererSelected); 
        if (!securityStatus.dlnaPlayOnEnabled) {
            playOnListPage.removeDLNASettings();
        }
    }

	function loadData(rendererTypes) {
	    //Only show HDMI if there is an HDMI port
        showHdmi = ((rendererTypes & ExternalRenderer.HDMI) && securityStatus.isHDMIPortAvailable());
        //Only show Miracast if the device supports it
        showMiracast = ((rendererTypes & ExternalRenderer.Miracast) && securityStatus.isMiracastAvailable());
        if (showMiracast) {
            // Removing Miracast from the renderer list, as we'll enable it if the 
            // user agrees to disable WiFiDirect profile or if WiFiDirect profile is alraedy disabled
            rendererTypes = (rendererTypes ^ ExternalRenderer.Miracast);
        }
        currentRendererTypes = (currentRendererTypes | rendererTypes);
        listView.dataModel.load(rendererTypes);
        showWiFiDirectDisableDialog();
    }
	
	function checkForDisabled() {
        // Need to check if the dialog is up so we can take it down
        if ((!showMiracast || !securityStatus.miracastPlayOnEnabled) && dialogUp) {
            console.log("Cancelling dialog");
            wifiDirectDisableDialog.cancel();
            dialogUp = false;
       }

        //Check to see if any play on types are available
	    //If there are no types available then pop the page
	    if (!securityStatus.dlnaPlayOnEnabled) {
	    	playOnListPage.removeDLNASettings();
	    	
	    	if ((!showHdmi || !securityStatus.hdmiPlayOnEnabled) &&
                (!showMiracast || isWiFiDialogSelectionCancelled() || !securityStatus.miracastPlayOnEnabled)) {
                if(!popping) {
                    //We need to avoid popping the page multiple times if multiple play on policies change at the same time
                    popping = true;
                    playOnListPage.navControl.pop();
                }	        	
	        }
	    } else {
	    	playOnListPage.addDLNASettings();
	    }
	}

	function showToast(msg) {
    	dialogUtils.showSystemToast(msg)
    }
    
    function showWiFiDirectDisableDialog() {
        console.log("showWiFiDirectDisableDialog");
        
        // If WiFiDirect profile is not set for Miracast then disable Miracast
        console.log("currentRendererTypes: " + (currentRendererTypes));
        if ((networkManager.wifiDirectEnabled == NetworkStatusManager.WD_Enabled ||
            networkManager.wifiDirectEnabled == NetworkStatusManager.WD_Unknown) && showMiracast && (currentRendererTypes & ExternalRenderer.Miracast)) {
            listView.dataModel.disable(ExternalRenderer.Miracast);
            currentRendererTypes = (currentRendererTypes ^ ExternalRenderer.Miracast);
        }
        
        if (securityStatus.miracastPlayOnEnabled && showMiracast) {
            if (networkManager.wifiDirectEnabled == NetworkStatusManager.WD_Enabled) {
                console.log("showWiFiDirectDisableDialog: showing dialog"); 
                dialogUp = true;
                getWifiDirectDisableDialog().show();
            } else if (networkManager.wifiDirectEnabled != NetworkStatusManager.WD_Unknown) {
                // Checking for only WD_Unknown here under the assumption that Miracast is functional
                // when WiFiDirect is (WD_DisabledByItPolicy || WD_NotSupported)
                console.log("WiFiDirect is ready, lets enable Miracast");
                if (dialogUp) {
                    console.log("Cancelling dialog because WFD is ready.");
                    wifiDirectDisableDialog.cancel();
                    dialogUp = false;
                }
                // Enable Miracast if we're not enabled
                if (!(currentRendererTypes & ExternalRenderer.Miracast)) {
                    listView.dataModel.enable(ExternalRenderer.Miracast);
                    currentRendererTypes = (currentRendererTypes | ExternalRenderer.Miracast);
                }
            }
        }
        else {
            console.log("Ignoring showing a wifi dialog or enabling Miracast.");
        }
    }
    
    function isWiFiDialogSelectionCancelled() {
        console.log("isWiFiDialogSelectionCancelled");
        // What was the last user selection
        // NOTE: dialog's result value gets reset to 0 when we call show()
        if (wifiDirectDisableDialog != null && wifiDirectDisableDialog.result == SystemUiResult.CancelButtonSelection) {
            console.log("Returning true");
            return true;
        }
        return false;
    }
    
    function getWifiDirectDisableDialog() {
        console.log("getWifiDirectDisableDialog");
        if (wifiDirectDisableDialog == null) {
            console.log("Creating new dialog");
            wifiDirectDisableDialog = dialogUtils.getSystemDialog();
            wifiDirectDisableDialog.title = qsTr("Wi-Fi Direct in use");
            wifiDirectDisableDialog.body = qsTr("To continue, the Wi-Fi Direct connection must be restarted and all devices disconnected. Do you want to continue?");
            wifiDirectDisableDialog.confirmButton.label = qsTr("Continue");
            wifiDirectDisableDialog.finished.connect(onWifiDirectDialogFinished);
        }
        return wifiDirectDisableDialog;
    }

    function onWifiDirectDialogFinished(result) {
        console.log("onWifiDirectDialogFinished: " + result);
        dialogUp = false;
        if (result == SystemUiResult.ConfirmButtonSelection && securityStatus.miracastPlayOnEnabled && showMiracast) {
            console.log("Selection confirmed!");
            // Enable Miracast if we're not enabled
            if (!(currentRendererTypes & ExternalRenderer.Miracast)) {
                listView.dataModel.enable(ExternalRenderer.Miracast);
                currentRendererTypes = (currentRendererTypes | ExternalRenderer.Miracast);
            }
        } else {
            console.log("Selection cancelled!");
            checkForDisabled();
        }
    }
    
    function checkWifiP2pRole() {
    	if (networkManager.wifiP2pRole == NetworkStatusManager.P2P_GO && showMiracast && securityStatus.miracastPlayOnEnabled) {
    		var miracastDevice = listView.dataModel.connected.name == undefined ? qsTr("the Miracast device that you're currently connected to") : listView.dataModel.connected.name;
            miracastGoWarning = qsTr("To discover other Miracast devices that you can play on, disconnect from %1.").arg(miracastDevice) + Retranslate.onLanguageChanged;
            showMiracastGoWarning = true;
    	} else {
            showMiracastGoWarning = false;
    	}
    }
	
	function checkHDMIPreventsMiracast() {
        if(listView.dataModel.connected.type == ExternalRenderer.HDMI && showMiracast && securityStatus.miracastPlayOnEnabled) {
            showNoMiracastHdmiConnectedWarning = true;
        } else {
            showNoMiracastHdmiConnectedWarning = false;
        }
	}
 
    ListView {
        id: listView
        signal rendererSelected(variant renderer)
        property SecurityStatus securityStatusInternal: securityStatus
        property NetworkStatusManager networkManagerInternal: networkManager
        property bool showMiracastInternal: showMiracast
        property bool showMiracastGoWarningInternal: showMiracastGoWarning
        property string miracastGoWarningInternal: miracastGoWarning
        property bool showNoMiracastHdmiConnectedWarningInternal: showNoMiracastHdmiConnectedWarning
        property int containerWidth: luh.layoutFrame.width

        scrollRole: ScrollRole.Main

        attachedObjects: [
            // Creating a LayoutHandler to handle width change when orientation changes
            LayoutUpdateHandler {
                id: luh
            }
        ]
        
        // Creating the PlayOnItemDataModel here to avoid any segfaults when 
        // this page is popped and the Model tries to do update ListView in its destructor
        dataModel: PlayOnItemDataModel {
            id: data
            
            onConnectedChanged: {
            	console.debug("PlayOnList.qml onConnectedChanged: " + connected);
                checkHDMIPreventsMiracast();
                checkWifiP2pRole();
            }
        }

        layoutProperties: StackLayoutProperties {
            spaceQuota: 100
        }
        
        layout: StackListLayout {
            headerMode: ListHeaderMode.Sticky
        }
        
        onTriggered:{
            // List Item tapped
            var selectedItem = listView.dataModel.data (indexPath);

            if (selectedItem.type == ExternalRenderer.Miracast && listView.dataModel.connected.type == ExternalRenderer.HDMI) {
                showToast(qsTr("Please unplug the HDMI cable before selecting a Miracast device to Play On.") + Retranslate.onLanguageChanged)
            } else  {
                if (!selectedItem.enabled && selectedItem.type == ExternalRenderer.HDMI) {
                    //:Message displayed when HDMI is not connected and user selectes HDMI from the list
                    showToast(qsTr("Connect your HDMI cable to use Presentation mode.") + Retranslate.onLanguageChanged)
                } else if (selectedItem.type !== undefined) {
                    // emit rendererSelected iff a valid listitem is selected e.g. where selectedItem.type == Hdmi, Miracast, Dlna
                    // and ignore other touch events from tapping the spinner/notification
                    rendererSelected(selectedItem);
                }
            }
        }
        
        listItemComponents: [
            ListItemComponent {
                type: "item"
                PlayOnListItem {
                    id: playOnListItem;
                    deviceTypeText: ListItemData.devicetypeinfo
                    deviceEnabled: ListItemData.enabled
                    showDisconnectButton: ListItemData.type == ExternalRenderer.Miracast
                } // PlayOnListItem
            },
            ListItemComponent {
                type: "header"
                Header {
                    title: ListItemData.name
                    opacity: 0.0
                    ListItem.onInitializedChanged: {
                        opacity = 0.0;
                        if (initialized) {
                            // If the item is being added to the list play the animation otherwise
                            // just hide it
                            headerFadeIn.play();
                        }
                    }
                    animations: [
                        FadeTransition {
                            id: headerFadeIn
                            duration: 1000
                            easingCurve: StockCurve.CubicOut
                            toOpacity: 1.0
                        }
                    ]
                } // Header
            },
            // To hide the headers
            ListItemComponent {
                type: ""
                Container {
                }
            },
            ListItemComponent {
                type: "hdmiitem"
                PlayOnListItem {
                    //:HDMI connection status
                    additionalText: ListItemData.enabled ? "" : "(" + (qsTr("Not Connected") + Retranslate.onLanguageChanged) + ")"
                    deviceTypeText: ListItemData.enabled ? ListItemData.devicetypeinfo : ""
                    deviceEnabled: ListItemData.enabled
                } // PlayOnListItem
            },
            ListItemComponent {
                type: "warning"
                Container {
                    //Warnings about finding (more) Miracast targets
                    id: warningContainer
                    background: MediaApplication.playOnBackgroundColor
                    minHeight: 120
                    opacity: 0.0
                    visible: warningContainer.ListItem.view.showMiracastGoWarningInternal || warningContainer.ListItem.view.showNoMiracastHdmiConnectedWarningInternal
                        
                    layout: DockLayout { }
                    
                    ListItem.onInitializedChanged: {
                        opacity = 0.0;
                        if (initialized) {
                            // If the item is being added to the list play the animation otherwise
                            // just hide it
                            itemFadeIn.play()
                        }
                    }

                    animations: [
                        FadeTransition {
                            id: itemFadeIn
                            duration: 1000
                            easingCurve: StockCurve.CubicOut
                            toOpacity: 1.0
                        }
                    ]

                    Container {
                        topPadding: 10
                        bottomPadding: 20
                        leftPadding: 20

                        verticalAlignment: VerticalAlignment.Center
                        horizontalAlignment: HorizontalAlignment.Fill

                        layout: StackLayout {
                            orientation: LayoutOrientation.TopToBottom
                        }
                        Label {
                            id: warningLabel
                            multiline: true
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            //We only have two different warnings and they cannnot be shown at the same time, and there is no need for that.
                            text: warningContainer.ListItem.view.showNoMiracastHdmiConnectedWarningInternal ? (qsTr("To discover Miracast devices that you can play on, unplug the HDMI cable.") + Retranslate.onLanguageChanged) : warningContainer.ListItem.view.miracastGoWarningInternal
                            textStyle.base: SystemDefaults.TextStyles.BodyText
                            textStyle.color: MediaApplication.textColor
                            textStyle.textAlign: TextAlign.Center
                        }
                    }
                        
                    Divider {
                        verticalAlignment: VerticalAlignment.Bottom
                    }
                }
            },
            ListItemComponent {
                type: "notification"
                //Container that displays a spinner and/or messages for exception scenarios (i.e. wifi off, list is empty, etc.)

                Container {
                    id: notificationContainer
                    layout: StackLayout {
                        orientation: LayoutOrientation.TopToBottom
                    }
                    Container {
                        preferredWidth: notificationContainer.ListItem.view.containerWidth
                        visible: indicator.visible
                        layout: DockLayout {
                        }
                        horizontalAlignment: HorizontalAlignment.Center

                        ActivityIndicator {
                            id: indicator
                            property bool networkActivity: ((notificationContainer.ListItem.view.networkManagerInternal.wifi == NetworkStatusManager.Connected || notificationContainer.ListItem.view.networkManagerInternal.wired == NetworkStatusManager.Connected) && notificationContainer.ListItem.view.securityStatusInternal.dlnaPlayOnEnabled)
                            property bool scanning: notificationContainer.ListItem.view.dataModel.scanning
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            running: indicator.networkActivity
                            visible: indicator.networkActivity
                            //preferredWidth: 100
                            preferredHeight: 100

                            onScanningChanged: {
                                // Show/hide the indicator only if the 30-second timer is inactive
                                if (scanning && ! timer.active) {
                                    if (fadeOut.isPlaying())  {
                                        fadeOut.stop();
                                    }
                                    indicator.visible = true;
                                    indicator.running = true;
                                } else if (! timer.active) {
                                    fadeOut.play();
                                }
                            }

                            onNetworkActivityChanged: {
                                if (networkActivity) {
                                    indicator.visible = true;
                                    indicator.running = true;
                                    timer.start();
                                } else {
                                    fadeOut.play();
                                }
                            }

                            attachedObjects: [
                                Timer {
                                    id: timer
                                    interval: 30000
                                    singleShot: true
                                    onTimeout: {
                                        // Don't hide the spinner if we're scanning
                                        if (! indicator.scanning) {
                                            fadeOut.play();
                                        }
                                    }
                                }
                            ]

                            animations: [
                                FadeTransition {
                                    id: fadeOut
                                    duration: 1000
                                    easingCurve: StockCurve.CubicOut
                                    toOpacity: 0.0
                                    onEnded: {
                                        // Show the spinner if we're still scanning, otherwise
                                        // just hide it
                                        if (indicator.scanning) {
                                            fadeIn.play();
                                        } else {
                                            indicator.running = false;
                                            indicator.visible = false;
                                        }
                                    }
                                    onStopped: {
                                        fadeIn.play();
                                    }
                                },
                                FadeTransition {
                                    id: fadeIn
                                    duration: 1000
                                    easingCurve: StockCurve.CubicOut
                                    toOpacity: 1.0
                                }
                            ] // Animations
                        } // ActivityIndicator
                    } // Container

                    Container {
                        visible: (wifiDialog.visible || noRenderersDialog.visible)
                        preferredWidth: notificationContainer.ListItem.view.containerWidth
                        layout: DockLayout {
                        }
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        rightPadding: 20
                        leftPadding: 20
                        bottomPadding: 10

                        Container {
                            layout: DockLayout {
                            }
                            minHeight: 300
                            visible: wifiDialog.visible && !MediaApplication.corporate
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            WifiOffDialog {
                                id: wifiDialog
                                networkManager: notificationContainer.ListItem.view.networkManagerInternal
                                securityStatus: notificationContainer.ListItem.view.securityStatusInternal
                                verticalAlignment: VerticalAlignment.Center
                                multipleTargetsPossible: notificationContainer.ListItem.view.showMiracastInternal &&
                                				notificationContainer.ListItem.view.securityStatusInternal.miracastPlayOnEnabled
                            }
                        }

                        Label {
                            id: noRenderersDialog
                            multiline: true
                            horizontalAlignment: HorizontalAlignment.Center
                            verticalAlignment: VerticalAlignment.Center
                            //If we don't have anything in the list and the wifi dialog is not visible, show the "No Renderers" text in the list
                            property bool dialogEnabled: notificationContainer.ListItem.view.dataModel.size == 0 && wifiDialog.visible == false
                            opacity: 0.0
                            visible: false
                            //:There are no renderers available in the Play On list
                            text: qsTr("No devices available to play on.") + Retranslate.onLanguageChanged
                            textStyle {
                                fontStyle: FontStyle.Italic
                                fontSize: FontSize.Medium
                                textAlign: TextAlign.Center
                            }

                            onDialogEnabledChanged: {
                                if (dialogEnabled) {
                                    noRenderersDialog.visible = true;
                                    noRenderersFadeIn.play();
                                } else {
                                    if (noRenderersFadeOut.state != AnimationState.Ended) {
                                        noRenderersFadeIn.stop();
                                    }
                                    noRenderersFadeOut.play();
                                }
                            }

                            animations: [
                                FadeTransition {
                                    id: noRenderersFadeOut
                                    duration: 1000
                                    easingCurve: StockCurve.CubicOut
                                    toOpacity: 0.0
                                    onEnded: {
                                        noRenderersDialog.visible = false;
                                    }
                                },
                                FadeTransition {
                                    id: noRenderersFadeIn
                                    duration: 1000
                                    easingCurve: StockCurve.CubicOut
                                    toOpacity: 1.0
                                }
                            ] // Animations
                        }
                    } // Container
                } // Outter Container
            }
        ]
    }
}
