import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * This control contains a shuffle, next, play, previous, repeat buttons
 * layed out horizontally.
 * @param shuffle_up_images Up Image locations for Shuffle button (OFF/ON)
 * @param shuffle_down_images Down Image locations for Shuffle button (OFF/ON)
 * @param repeat_up_images Up Image locations for Repeat button (OFF/ON/ONE)
 * @param repeat_down_images Down Image locations for Repeat button (OFF/ON/ONE)
 * @param repeat_down_images Down Image locations for Repeat button (OFF/ON/ONE)
 * @param play_pause_up_images Up Image locations for Play/Pause button (Play/Pause) 
 * @param play_pause_down_images Down Image locations for Play/Pause button (Play/Pause)
 * @param next_up_image Up Image locations for Next button 
 * @param next_down_image Down Image locations for Next button 
 * @param previous_up_image Up Image locations for Previous button 
 * @param previous_down_image Down Image locations for Previous button 
 * @param shuffle_visible Controls whether Shuffle button will be displayed
 * @param repeat_visible Controls whether Repeat button will be displayed
 * @param topPadding Sets Top Padding for Player Controls
 * @param bottomPadding Sets Bottom Padding for Player Controls
 * @signal nextPressed	Emitted when next is pressed
 * @signal previousPressed Emitted when previous is pressed
 * @signal shufflePressed Emitted when shuffle is pressed
 * @signal playPausePressed Emitted when play/pause button is pressed
 * @signal repeatPressed Emitted when repeat button is pressed
 * @slot onPlaying signifies that current playback state is playing
 * @slot onPaused signifies that current playback state is paused
 * @slot onStopped signifies that current playback state is stopped
 * @slot enableNext signifies a request to enable Next button
 * @slot disableNext signifies a request to disable Next button
 * @slot enablePrevious signifies a request to enable Previous button
 * @slot disablePrevious signifies a request to disable Previous button
 * @slot onRepeatModeChanged(mode) signifies that repeat mode has changed
 * @slot onShuffleEnabled signifies that shuffle has been enabled
 * @slot onShuffleDisabled signifies that shuffle has been disabled
 */

Container {
    id: playerControls
    property alias shuffle_up_images: shuffle.upImages
    property alias shuffle_down_images: shuffle.downImages
    property alias repeat_up_images: repeat.upImages
    property alias repeat_down_images: repeat.downImages
    property alias previous_up_image: previous.upImages
    property alias previous_down_image: previous.downImages
    property alias next_up_image: next.upImages
    property alias next_down_image: next.downImages
    property alias play_pause_up_images: playPause.upImages
    property alias play_pause_down_images: playPause.downImages
    property alias shuffle_visible: shuffle.visible
    property alias repeat_visible: repeat.visible

    signal nextPressed
    signal previousPressed
    signal playPausePressed
    signal shufflePressed
    signal repeatPressed
    
    horizontalAlignment: HorizontalAlignment.Center
    verticalAlignment: VerticalAlignment.Center
    
    layout: StackLayout {
        id: playerControlsRootLayout
        orientation: LayoutOrientation.LeftToRight
    }
    
    // shuffle button 
    MultiStateImageView {
        id: shuffle
        property bool isShuffled: false
        
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Fill
        
        layoutProperties: StackLayoutProperties {
            id: stackLayoutPropertiesButton
            spaceQuota: 20            
        }
        onTouch: {
            if (event.isUp () && enabled) {
                playerControls.shufflePressed ();
            }
        }
        onIsShuffledChanged: {
            if (isShuffled) {
                currentState = MusicPlaybackController.ShuffleOn;
            } else {
                currentState = MusicPlaybackController.ShuffleOff;
            }
        }
    }          
            
    // previous button
    MultiStateImageView {
        id: previous
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        layoutProperties: stackLayoutPropertiesButton
        onTouch: {
            if (event.isUp ()) {
                playerControls.previousPressed ();
            }
        }
    } 
    
    
    
    // play/pause button
    MultiStateImageView {
        id: playPause
        property bool isPlaying: false
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        layoutProperties: stackLayoutPropertiesButton
        onTouch: {
            if (event.isUp () && enabled) {
                playerControls.playPausePressed ();
            }
        }
        onIsPlayingChanged: {
            if (isPlaying) {
                playPause.currentState = PlaybackController.PlaybackPlaying;
            } else {
                playPause.currentState = PlaybackController.PlaybackPaused;
            }
        }
    }               
    
    // next button
    MultiStateImageView {
        id: next
        horizontalAlignment: HorizontalAlignment.Center
        verticalAlignment: VerticalAlignment.Center
        layoutProperties: stackLayoutPropertiesButton
        onTouch: {
            if (event.isUp ()) {
                playerControls.nextPressed ();
            }
        }
    }                
    
    // repeat button//PlaybackController.RepeatOff
    MultiStateImageView {
        id: repeat
        property int repeatMode: MediaPlayer.None
        horizontalAlignment: playerControls.horizontalAlignment
        verticalAlignment: playerControls.verticalAlginment
        layoutProperties: stackLayoutPropertiesButton
        onTouch: {
            if (event.isUp () && enabled) {
                playerControls.repeatPressed ();
            }
        }
        onRepeatModeChanged: {
            currentState = repeatMode;
        }
    }
    function onPlaying () {
        playPause.isPlaying = true;
    }
    function onStopped () {
        playPause.isPlaying = false;
    }
    function onPaused () {
        playPause.isPlaying = false;
    }
    function enableNext () {
        next.enabled = true;
        next.opacity = 1.0;
    }
    function disableNext () {
        next.enabled = false;
        next.opacity = 0.3;
    }
    function enablePrevious () {
        previous.enabled = true;
        previous.opacity = 1.0;
    }
    function disablePrevious () {
        previous.enabled = false;
        previous.opacity = 0.3;
    }
    function onRepeatModeChanged (mode) {
        repeat.repeatMode = mode;
    }
    function onShuffleEnabled () {
        shuffle.isShuffled = true;
    }
    function onShuffleDisabled () {
        shuffle.isShuffled = false;
    }
}
