import bb.cascades 1.2
import bb.cascades.multimedia 1.0

Container {
    id: propertyFileNameContainer
    property alias title : titleLabel.text
    property string fileName: ListItemData.fileName
    property bool forceReadOnly: false
    
    property int height;
    
    minHeight: height

    layout: DockLayout {
    }

	Container {
	    leftPadding: 20
	    rightPadding: 20
        bottomPadding: ListItemData.isWritable ? 20 : 10
	
	    horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment:  VerticalAlignment.Center
	    
	    layout: StackLayout {
	        orientation: LayoutOrientation.TopToBottom
	    }
		
	    Label {
	        id: titleLabel
	        horizontalAlignment: HorizontalAlignment.Fill
	        textStyle {
	            color: Color.Gray
	            fontSize: FontSize.Small
	            fontWeight: FontWeight.W400
	        }
            topMargin: 0
            bottomMargin: 0
	    }
	
		// Visibility of the Label (fileNameLabel)
		// and the TextArea (fileNameTextField) are mutually exclusive.
	    Label {
	        id: fileNameLabel
	        text: fileName
	        visible: !ListItemData.isWritable || forceReadOnly
	        multiline: true
	        horizontalAlignment: HorizontalAlignment.Fill
	        textStyle {
	            fontSize: FontSize.Large
	            fontWeight: FontWeight.W400
	        }
            topMargin: 0
            bottomMargin: 0
	    }
	    TextField {
	        id: fileNameTextField
	        text: fileName
	        hintText: qsTr("Enter file name");
	        enabled: ListItemData.isWritable && !forceReadOnly
	        visible: ListItemData.isWritable && !forceReadOnly
	        inputMode: TextAreaInputMode.Text
	        content.flags: TextContentFlag.EmoticonsOff | TextContentFlag.ActiveTextOff;
	        horizontalAlignment: HorizontalAlignment.Fill	        
            topMargin: 10 
            validator: Validator {
            	id: renameValidator
            	mode: ValidationMode.Immediate
            	onValidate: {
            		var attemptedRename = fileNameTextField.text.trim()+"."+fileNameTextField.extension;
            		// attemptedRename: When the Text Field is in focus, the file extension is not displayed, thus the 
                    // renamed file must have the extension added
                    var attemptedRenameNoExtensionAdded = fileNameTextField.text.trim();
                    // attemptedRenameNoExtensionAdded: When the Text Field is not in focus, the file extension is
                    // already included in the fileNameTextField
                    if (attemptedRename == fileName) { 
                        // This condition occurs when the Text Field is in focus and the file name has not been changed
                        // from the original. No validation is needed in this case. Since the field is in focus, 
                        // attemptedRename is used for comparison
                        return ValidationState.Unknown;
                    } 
                    if (attemptedRenameNoExtensionAdded == fileName) {
                        // This condition occurs when the Text Field is not in focus. No validation is needed in this case. 
                        // Since the field is not in focus, attemptedRenameNoExtensionAdded is used for comparison
                        return ValidationState.Unknown;
                    }
                    if (attemptedRenameNoExtensionAdded == ""){
                        // When no name is in the text field, the hint text appears, and the validator is unnecessary
                        return ValidationState.Unknown;
                    }
                    // In the following cases, an attempt to rename the file has been made, and the Text Field is in focus.
                    // In each cases the validator is used to show the user if the name is valid as they update the text field
            		var reason = propertyFileNameContainer.ListItem.view.dataModel.validateRename(attemptedRename);
            		if (reason == PropertyError.Success) {
            			renameValidator.state = ValidationState.Valid;
            		} else {
            			renameValidator.state = ValidationState.Invalid;
            			
            			if (reason == PropertyError.InvalidCharacters) {
            				renameValidator.errorMessage = qsTr("The new file name has invalid characters");
            			} else if (reason == PropertyError.FileExists) {
            				renameValidator.errorMessage = qsTr("The new file name already exists.");
            			}
            		}
            	}
            } 
	
	        input {
	        	submitKey: SubmitKey.Done;
	            onSubmitted: {
	                loseFocus();
	            }
	     
	            flags: TextInputFlag.SpellCheckOff | TextInputFlag.PredictionOff | TextInputFlag.AutoCorrectionOff | TextInputFlag.AutoCapitalizationOff | TextInputFlag.AutoPeriodOff | TextInputFlag.WordSubstitutionOff
	        }
	
	        textStyle {
	            fontSize: FontSize.Large
	            fontWeight: FontWeight.W400
	        }
	
			property string originalFileName: ""
	        property string extension: "" 
	        onFocusedChanged: {
	            if (focused) {
	                var lastDotIndex = fileName.lastIndexOf(".");
	                var baseName = fileName.substring(0, lastDotIndex);
	                extension = fileName.substring(lastDotIndex + 1);
	
	                fileNameTextField.text = baseName;
	            } else {
	            	var newFileName = fileNameTextField.text.trim();
	                if (newFileName.length < 1) {
	                    newFileName = fileName;
	                } else {
	                    if (extension.length > 0) {
	                        newFileName += "." + extension;
	                    }
	                }
	
	                if (newFileName != fileName) {
	                    var fileRenamed = propertyFileNameContainer.ListItem.view.dataModel.renameFile(newFileName);
	                    if (fileRenamed == PropertyError.Success) {
	                        commonAnalytics.instance.logFileNameChanged(fileName, newFileName);
	                        propertyFileNameContainer.ListItem.view.propertiesPage.addAction(discardChangesAction);
	                        originalFileName = fileName;
	                        fileName = newFileName;
	                    } else if (fileRenamed == PropertyError.InvalidCharacters) {
	                    	dialogUtils.showSystemToast(qsTr("The new file name has invalid characters"));
	                    } else if (fileRenamed == PropertyError.FileExists) {
	                    	dialogUtils.showSystemToast(qsTr("The new file name already exists."));
	                    } else {
	                        dialogUtils.showSystemToast(qsTr("This file can't be renamed."));
	                    }
	                } 
	                fileNameTextField.text = fileName;
	        	} 
	        } 
	        attachedObjects: [
	            ActionItem {
	                id: discardChangesAction
	                title: qsTr("Discard Changes") + Retranslate.onLanguageChanged
	                onTriggered: {
                        if (propertyFileNameContainer.ListItem.view.dataModel.renameFile(fileNameTextField.originalFileName) != PropertyError.Success) {
	                        dialogUtils.showSystemToast(qsTr("Can't revert to old file name."));
	                    } else {
	                    	fileName = fileNameTextField.originalFileName;
	                        fileNameTextField.text = fileNameTextField.originalFileName;
	                    }
	
	                    propertyFileNameContainer.ListItem.view.propertiesPage.removeAction(discardChangesAction);
	                }
	                ActionBar.placement: ActionBarPlacement.OnBar
	                imageSource: "images/ic_discard_changes.png"
	            },
	            SystemDialogUtils{
	                id: dialogUtils
	            },
	            CommonAnalytics {
	                id: commonAnalytics
	            }
	        ]
	    }
    }

    Divider {
        verticalAlignment: VerticalAlignment.Bottom
    }
}
