import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * A search screen that contains:
 *   - A SearchControl with a text field and a Cancel button.
 *   - A list view that displays the search result.
 *     The list view is assigned by setting by setting searchListViewDef property.
 *
 *     For example:
 *         attachedObjects:[
 *             ComponentDefinition {
 *                 id: mySearchListViewDef
 *                 ListView {
 *                     id: myListView
 *                     dataModel: myDataModel
 *                 }
 *             }
 *         ]
 * 
 *         SearchScreen {
 *             id: musicSearchScreen
 *             searchListViewDef: musicSearchListViewDef
 *         }
 */
Container {
    id: searchScreen
    
    property ComponentDefinition searchListViewDef // assign this property to the Component Definition
                                                   // of the list view where the search result
                                                   // shall be displayed
    property string noContentLabel
    property string hintText: qsTr("Search") + Retranslate.onLanguageChanged

    property alias searchListView: listViewContainer.control
    property string __state

    signal searchScreenDisplayed()
    signal searchScreenCancelled()
    signal searchTextChanged(string text) //used to propagate signal from SearchControl

    function modelsChanged(){
        searchControls.handleModelsChanged();
    }
    
    layout: DockLayout {
    }
    
    Container{
        
        id: searchControls
        	  
	    layout: StackLayout  {
            orientation: LayoutOrientation.TopToBottom
        }

        signal setModelCriteria(variant criteria)
        signal applyModelFilter()
        
	    function handleModelsChanged(){
            var headerCount = searchListView.dataModel.childCount(null);
            var searchText = searchControl.getSearchText();
            if(headerCount == 0 ){
                if(searchText.length == 0){
                    searchControls.setSearchState("no_query");
                } else {
                    searchControls.setSearchState("no_results");
                }   
             } else {
                 searchControls.setSearchState("search_completed");
             } 	
            
            if(searchControl.finalSearch){
                searchControl.finalSearch = false;
                searchControl.searchTextChanged(searchText);
            }
	    }
	           
	    function performSearch(criteria){
	        setSearchState("searching");
	        setModelCriteria(criteria);
	        applyModelFilter();
	    }
	    
	    function searchCancelled(){
	        console.debug("SearchScreen::searchCancelled");
	        searchControl.clearText();
            searchScreen.searchScreenCancelled();
	    }
	    
	    function setSearchState(state){
            console.debug("Search state: " + state);
            __state = state;
            if(state == "searching"){
                searchLabel.visible  = false;
                searchIndicator.start();
            } else if(state == "search_completed"){
                searchLabel.visible  = false;
                searchIndicator.stop();
            } else if(state == "no_query"){
                searchLabel.visible  = true;
                searchIndicator.stop();
            } else if(state == "no_results"){
                searchLabel.visible  = true;
                searchIndicator.stop();
            }
	    }	 
	  
	    
	    onCreationCompleted: {
            searchListView.dataModel.relay.connect(searchControls, "setModelCriteria(QVariant)" ,"setCriteria(QVariant)");
	        searchListView.dataModel.relay.connect(searchControls, "applyModelFilter()" ,"applyFilter()");
	        searchControl.searchTextChanged.connect(performSearch);
	        searchControl.searchCancelled.connect(searchCancelled);
    
	        setSearchState("no_query");
        }

	    SearchControl {
	        id: searchControl
	        backgroundColor: MediaApplication.searchScreenBackgroundColor
	        buttonText: qsTr("Cancel") + Retranslate.onLanguageChanged
	        hint: searchScreen.hintText
			onSearchTextChanged: {
                searchScreen.searchTextChanged(text)
            }
        }

		Container {
            layoutProperties: StackLayoutProperties {
                spaceQuota: 90
            }
            layout: DockLayout {
            }
            Container {
                layout: DockLayout {
                }
                Container {
                    id: labelContainer
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Fill
                    layout: DockLayout {
                    }
                    Label {
                        id: searchLabel
                        textStyle.base: SystemDefaults.TextStyles.BodyText
                        textStyle.color: MediaApplication.textColor
                        textStyle.textAlign: TextAlign.Center
                        text: {
                            __state == "no_query" ? qsTr("Type to search.") + Retranslate.onLanguageChanged : 
                            __state == "no_results" ? noContentLabel : ""
                        }
                        horizontalAlignment: HorizontalAlignment.Center
                        verticalAlignment: VerticalAlignment.Center
                        multiline: true   
                    }
                }
                
                ControlDelegate {
                    id: listViewContainer
                    horizontalAlignment: HorizontalAlignment.Fill
                    verticalAlignment: VerticalAlignment.Fill
                    sourceComponent: searchListViewDef
                }
            }
            //search activity indicator
            ActivityIndicator {
                id: searchIndicator
                preferredWidth: 256
                preferredHeight: 256
                
                visible: running
                
                overlapTouchPolicy: OverlapTouchPolicy.Allow
                horizontalAlignment: HorizontalAlignment.Center
                verticalAlignment: VerticalAlignment.Center
            }
		}
	}

    function setAlwaysFocused(alwaysFocused) {
        searchControl.setAlwaysFocused(alwaysFocused);
    }

    function focusSearchField(focused) {
        searchControl.focus(focused);
    }
    
    function enableVirtualKeyboard(enabled){
        searchControl.enableVirtualKeyboard(enabled);
    }
    
    function displaySearchResult(visible) {
        labelContainer.visible = visible;
        searchListView.visible = visible;
    }
}
