import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0

/**
 * This control is a progress slider bar that includes a slider and
 * text for both the current time and remaining time.
 *
 * @param     textColor        Color of the text
 * @param     controlsRightMargin  Right margin to set on the inner controls
 * @param     controlsLeftMargin   Left margin to set on the inner controls
 * @param     controlsTopMargin  Top margin to set on the inner controls
 * @param     controlsBottomMargin   Bottom margin to set on the inner controls
 * @param     controlsSpacingMargin  Space between the slider and values    
 * @param     sliderBarValue   position of the slider
 * @param     sliderBarToValue  length of the slider bar
 * @param     hidden Parameter to signify slider is not visible and should not updated value
 
 * @signal    valueChanging   emitted when the value is changing
 */

Container {
    id: sliderBox
    attachedObjects: [
        Timer {
            id: timer
            singleShot: true
        }
    ]
    property variant elapsedTimeTextColor: MediaApplication.timeSliderTextColor
    property variant durationTextColor: MediaApplication.timeSliderTextColor
    property variant elapsedTimeFont: SystemDefaults.TextStyles.SubtitleText
    property variant durationFont: SystemDefaults.TextStyles.SubtitleText
    property variant timeSliderControllerRef
    property bool updateOnSeek: true
    property bool hidden
    property alias controlsLeftPadding: sliderBox.leftPadding
    property alias controlsRightPadding: sliderBox.rightPadding
    property alias controlsTopPadding: sliderBox.topPadding
    property alias controlsBottomPadding: sliderBox.bottomPadding
    property alias controlsInterSpacing: sliderLabels.topPadding

    property alias controlsSpacingMargin: sliderBar.bottomMargin
    property alias sliderBarValue: sliderBar.value
    property alias sliderBarToValue: sliderBar.toValue
    property bool __userDragged: false
    property int sliderFinalPosition: -1
    property int lastReportedPosition
    
    signal valueChanging (real value)
    signal seekingStarted ()
    signal seekingStopped ()
    layout: StackLayout {
        id: stackLayout
        orientation: LayoutOrientation.TopToBottom
    }
    onCreationCompleted: {
        enableSeek(timeSliderControllerRef.seekable);
        updateSliderDuration(timeSliderControllerRef.duration);
        updateSliderPosition(timeSliderControllerRef.position);
        timeSliderControllerRef.seekableChanged.connect(enableSeek);
        timeSliderControllerRef.durationChanged.connect (updateSliderDuration);
        timeSliderControllerRef.positionChanged.connect (updateSliderPosition);
        timeSliderControllerRef.timeToUpdateSlider.connect (setSliderFinalPosition);
        timer.timeout.connect (resetUserDragged);
    }
    
    onHiddenChanged: {
        if (!hidden) {
            sliderBar.value=lastReportedPosition
        }
    }
    
    function disconnectSlots()
    {
        timeSliderControllerRef.seekableChanged.disconnect(enableSeek);
        timeSliderControllerRef.durationChanged.disconnect (updateSliderDuration);
        timeSliderControllerRef.positionChanged.disconnect (updateSliderPosition);
        timeSliderControllerRef.timeToUpdateSlider.disconnect (setSliderFinalPosition);
    }
    
    function saveSliderPosition (value) {
        //console.debug("saveSliderPosition() in qml: value = " + value)
        timeSliderControllerRef.saveSliderPosition (value)
    }
    function setSamplingMode (start) {
        if (start) {
            timer.timeout.disconnect (resetUserDragged);
            if (timer.active) {
                timer.stop ();
            }
            __userDragged = true;
            sliderFinalPosition = -1; //if the last final position has not been set, reset it
        }
        // Don't start the seek update position timer if updateOnSeek == false.  
        // We do however, in all cases want to call setSamplingMode(false);
        if (updateOnSeek || ! start) timeSliderControllerRef.setSamplingMode (start)
    }
    function setPlayerPosition (value) {
        if (__userDragged) {
            timeSliderControllerRef.position = value;
            //__userDragged = false
        }
    }
    function enableSeek(seekable) {
        timeSliderControllerRef.printInfo("+enableSeek");
        console.debug("enableSeek() in qml: seekable = " + seekable);
        sliderBar.seekable = seekable;
        timeSliderControllerRef.printInfo("-enableSeek");
    }
    
    function updateSliderDuration (value) {
        timeSliderControllerRef.printInfo("+updateSliderDuration");
        sliderBar.toValue = value;
        //Make sure we update the slider position if we've got a real duration and we're paused
        //We can't do this while we're playing or we'll see the slider jumping back and forth
        //because on a next, duration gets updated before the position so we'd be updating the
        //slider with the position of the old track not the new one.
        if(value != 0 && timeSliderControllerRef.playbackControllerRef.playbackState == PlaybackController.PlaybackPaused) {
            updateSliderPosition(timeSliderControllerRef.position);
        }
        timeSliderControllerRef.printInfo("-updateSliderDuration");
    }

    function updateSliderPosition (value) {
        timeSliderControllerRef.printInfo("+updateSliderPosition");
        try {
	        if (! sliderBar.isSeeking && ! __userDragged) {
	            lastReportedPosition = value
	            if ( !hidden) {
	                if (sliderBar.toValue != 0) {
						// We have a duration, set the value on the slider
	                	sliderBar.value = value;
	            	} else {
	            		// If we have no duration, setting the Slider value, won't have any effect.
	            		// Since we can't rely on the Slider to give us time updates, we need to
	            		// set the time text label directly here.
	            		var roundedValue = (value.toFixed (1) + 500 ) / 1000; //rounded to seconds
	            		time.text = localeUtils.getColonSeparatedDuration (roundedValue.toFixed (0) * 1000);
	            		duration.text = "-" + localeUtils.getColonSeparatedDuration (0);
	            	}
	            }
	        }
         }
         catch (e) {
             timeSliderControllerRef.printInfo("TimeSlider.qml::updateSliderPosition() - exception caught, likely due to current source component. Exception: " + e);
         }
         timeSliderControllerRef.printInfo("-updateSliderPosition");
    }

    function setSliderFinalPosition () {
        var roundedToValue = (sliderBar.toValue.toFixed (1) + 500 ) / 1000 ; //rounded to seconds
        var roundedValue = (sliderBar.value.toFixed (1) + 500 ) / 1000; //rounded to seconds
        time.text = localeUtils.getColonSeparatedDuration (roundedValue.toFixed (0) * 1000);
        duration.text = "-" + localeUtils.getColonSeparatedDuration ((roundedToValue.toFixed (0) - roundedValue.toFixed (0)) * 1000);
        timer.timeout.disconnect (resetUserDragged);
        if (timer.active) {
            timer.stop ();
        }
        timer.timeout.connect (resetUserDragged);
        timer.start (500);
        if (__userDragged && sliderFinalPosition != -1) {
            setPlayerPosition (sliderFinalPosition);
            sliderFinalPosition = -1;
        }
    }
    function resetUserDragged () {
        __userDragged = false;
    }
    Slider {
        id: sliderBar
        objectName: "sliderBar"                
        property bool isSeeking: false
        property bool isNewPositionSet: false
        
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Center
        
		// True if the stream is seekable
        property bool seekable: false

        // Only enable the slider if we have a duration (i.e. the user will be able to seek) 
		// and if the stream is seekable.
        enabled: toValue != 0 && seekable

        fromValue: 0
        toValue: 0
        
        onCreationCompleted: {
            seekable = timeSliderControllerRef.seekable;
        }
        
        onImmediateValueChanged: {
            var roundedToValue = (toValue.toFixed (1) + 500 ) / 1000 ; //rounded to seconds
            var roundedValue = (immediateValue.toFixed (1) + 500 ) / 1000; //rounded to seconds
            time.text = localeUtils.getColonSeparatedDuration (roundedValue.toFixed (0) * 1000);
            duration.text = "-" + localeUtils.getColonSeparatedDuration ((roundedToValue.toFixed (0) - roundedValue.toFixed (0)) * 1000); 
            if (isSeeking) {                
                sliderBox.valueChanging (immediateValue);
                saveSliderPosition (immediateValue);
            }
        }
            

        onValueChanged: {
            if (__userDragged) {
                sliderFinalPosition = value;
            }
            /*var roundedToValue = ( toValue.toFixed(1) + 500 ) / 1000 ; //rounded to seconds
            var roundedValue = ( value.toFixed (1) + 500 ) / 1000; //rounded to seconds
            time.text = localeUtils.getColonSeparatedDuration( roundedValue.toFixed(0) * 1000 ); 
            duration.text = "-" + localeUtils.getColonSeparatedDuration( (roundedToValue.toFixed(0) - roundedValue.toFixed(0)) * 1000 ); 
            setPlayerPosition(value);*/
        }
        onTouch: {
            if (event.isDown ()) {
                isSeeking = true
                sliderBox.seekingStarted ();
                setSamplingMode (true)
            } else if (event.isUp () || event.isCancel ()) {
                isSeeking = false
                sliderBox.seekingStopped ();
                setSamplingMode (false)
            }
        }
    }
    Container {
        id: sliderLabels
        layout: DockLayout {
        }
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Center
        leftPadding: 20
        rightPadding: 20

        Label {
            id: time
            
            horizontalAlignment: HorizontalAlignment.Left
            text: ""
            textStyle.color: sliderBox.elapsedTimeTextColor
            textStyle.base: sliderBox.elapsedTimeFont
        }
        Label {
            id: duration
            
            horizontalAlignment: HorizontalAlignment.Right

            text: ""
            textStyle.color: sliderBox.durationTextColor
            textStyle.base: sliderBox.durationFont
        }
    }
}
