import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0


/*
* This defines a component to be used as the visual for
* a volume control slider with images on the left and right
*
* @param leftImageSpeaker The image for left icon when volume level is > 0
* @param leftImageMute The image for the left icon when volume level is 0
* @param rightImageSpeaker The image for the right icon when volume is below headphoneOverrideLevel
* @param rightImageBoost The image for the right icon when volume is above headphoneOverrideLevel
* @param increaseImage     The url for the image used when the increase button is not being pressed
* @param decreaseImage     The url for the image used when the decrease button is not being pressed
* @signal increaseVolume()         Emitted when volume is increased 
* @signal decreaseVolume()         Emitted when volume is decreased 
* @signal volumeLevelChanging emitted as the volume level is being changed
* @signal leftImageTouchUp emitted when the left image is touched
* @signal rightImageTouchUp emitted when the right image is touched
*/
Container {

    id: volumeControlContainer
    
    //The playback controller to be used for getting/setting volume
    //This will be set when the screen which uses this control sets it via connectVolumePlaybackController
    property variant volumePlaybackController
    property bool isDLNA: false
   
    property alias volumeTopMargin: volumeContainer.topPadding
    property alias volumeBottomMargin: volumeContainer.bottomPadding
    property alias volumeBackground: volumeContainer.background

    property string leftImageSpeaker
    property string leftImageMute
    property string rightImageSpeaker
    property string rightImageBoost
    property int volumeControlType: AudioDevice.Percentage
    property bool headphoneOverrideEnabled: false
    property int headphoneOverrideLevel: 100 //The volume percentage which we represent audio boost or unsafe zone is enabled

    property bool __volumeChangedByUser: false  

    property alias increaseImage: increaseButton.imageSource
    property alias decreaseImage: decreaseButton.imageSource

    signal volumeLevelChanging( int value )
    signal leftImageTouchUp()
    signal rightImageTouchUp()
    signal increaseVolume()
    signal decreaseVolume()

    layout: DockLayout {
    }
   
    attachedObjects: [
        Timer{
            id: timer
            singleShot: true                    
        },
        HeadphoneAudioDevice{
            id: headphoneAudioDevice
            onAudioBoostChanged:{
                console.debug("onAudioBoostChanged " + enabled);
                headphoneOverrideEnabled = enabled;
                headphoneOverrideLevel = headphoneAudioDevice.audioBoostLevel;
                setVolumeIcon(sliderBar.value);
            }
            onUnsafeZoneChanged:{
                console.debug("onUnsafeZoneChanged " + enabled);
                headphoneOverrideEnabled = enabled;
                headphoneOverrideLevel = headphoneAudioDevice.unsafeZoneLevel;
                setVolumeIcon(sliderBar.value);
            }
        },
        AudioManager {
            id: audioManager
        },
	CommonAnalytics {
	    id: commonAnalytics
	}
    ]

    onCreationCompleted: {
        timer.timeout.connect( resetVolumeChangedByUser );
        if (headphoneAudioDevice.audioBoostSupported) {
            headphoneOverrideEnabled = headphoneAudioDevice.audioBoostEnabled;
	        headphoneOverrideLevel = headphoneAudioDevice.audioBoostLevel;
	    } else if (headphoneAudioDevice.unsafeZoneSupported) {
	        headphoneOverrideEnabled = headphoneAudioDevice.unsafeZoneEnabled;
	        headphoneOverrideLevel = headphoneAudioDevice.unsafeZoneLevel;
	    }
        setVolumeIcon(sliderBar.value);
    }
    
    function setVolumeControlType(newDeviceType, newVolumeControlType, initialVolume) {
        volumeControlType = newVolumeControlType;
        updateVolumeSlider( initialVolume );
    }
    
    function connectVolumePlaybackController(aPlaybackController, isRemotePlaybackController) {
        if(volumePlaybackController != aPlaybackController) {
            console.debug("VolumeControl.connectVolumePlaybackController: changing from: " + volumePlaybackController + " to " + aPlaybackController);
            if ( volumePlaybackController != null ){
                volumePlaybackController.activeAudioDeviceChanged.disconnect( setVolumeControlType );
                volumePlaybackController.volumeChanged.disconnect(updateVolumeSlider);
            }
            volumePlaybackController = aPlaybackController;
            isDLNA = isRemotePlaybackController;
            volumePlaybackController.volumeChanged.connect(updateVolumeSlider);
            if ( isDLNA ) {
                volumeControlType = AudioDevice.Percentage;
            } else {
                volumeControlType = audioManager.activeAudioDevice.volumeControlType;
                volumePlaybackController.activeAudioDeviceChanged.connect( setVolumeControlType );
            }
            if ( volumeControlType == AudioDevice.Percentage ) {
                updateVolumeSlider( volumePlaybackController.volume );  
            }            
        } else {
            console.debug("VolumeControl.connectVolumePlaybackController: staying with: " + volumePlaybackController);
        }
        
    }
    
    function disconnectVolumePlaybackController()
    {
        volumePlaybackController.activeAudioDeviceChanged.disconnect( setVolumeControlType );
        volumePlaybackController.volumeChanged.disconnect(updateVolumeSlider);
    }
	    
    function saveVolume(value){
        volumePlaybackController.saveVolume(value);
    }

    function setVolume(value){
        console.debug("setVolume() in qml: changedByuser? " + __volumeChangedByUser );
        if( __volumeChangedByUser ){
            console.debug("setVolume() in qml: value = " + value );    
            volumePlaybackController.volume = value;
            //__volumeChangedByUser = false;
        }
    }

    function resetVolumeChangedByUser(){
        __volumeChangedByUser = false;    
    }

    function setVolumeMode( start ){    
        console.debug("setVolumeMode() in qml: start = " + start );    
        if( start ){
            timer.timeout.disconnect( resetVolumeChangedByUser );
            if(timer.active){
                timer.stop();
            }
            volumePlaybackController.saveVolume(sliderBar.value);
            __volumeChangedByUser = true;    
        }else{
            timer.timeout.connect( resetVolumeChangedByUser );
            timer.start(200);
        }
        // Don't update the volume while moving the slider for remote playback.
        if ( !isDLNA) {
            volumePlaybackController.setVolumeMode(start);
        }
    }

    function setVolumeChangedByUser(){
        __volumeChangedByUser = true;    
    }

    function updateVolumeSlider(value){
        if( !__volumeChangedByUser && !sliderBar.isSeeking && volumeControlType == AudioDevice.Percentage ){
            console.debug("updateVolumeSlider() in qml: value = " + value );
            sliderBar.value = value; 
        } else {
            console.debug("updateVolumeSlider() in qml: IGNORING value = " + value);
        }
    }
    
    function setLeftImageVisible(value)
    {
        left.visible = value;
    }

    function setVolumeIcon(value) {
        //update the volume icon to off if the volume is set to 0
        if (value.toFixed(1) == 0) {
            leftIcon.imageSource = leftImageMute
        } else {	//we have volume so have the icon indicate on
            leftIcon.imageSource = leftImageSpeaker
        }
        
        if (headphoneOverrideEnabled && value.toFixed(1) > headphoneOverrideLevel && (audioManager.activeAudioDevice.deviceType == AudioDevice.Headphone || audioManager.activeAudioDevice.deviceType == AudioDevice.Headset)) {
            //headphone override is enabled, so show the red icon
            rightIcon.imageSource = rightImageBoost
        } else {
            rightIcon.imageSource = rightImageSpeaker
        }
    }


    Container{
        id: volumeContainer
        
        horizontalAlignment: HorizontalAlignment.Fill             
        verticalAlignment: VerticalAlignment.Fill
        visible: volumeControlType == AudioDevice.Percentage
        
        topPadding: 34
        bottomPadding: 35
        leftPadding: 18
        rightPadding: 19
        background: MediaApplication.volumeControlBackgroundColor

        layout: StackLayout {
            orientation: LayoutOrientation.LeftToRight
        }

        //Image view to hold the volume on/off icon
        ImageView {
            id: leftIcon
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Left

             onTouch: {
                if (event.isUp()) {
                    timer.timeout.disconnect( resetVolumeChangedByUser );
                    if(timer.active){
                        timer.stop();
                    }
                    //console.debug("isUp onTouch in volumecontrol.qml")                
                    sliderBar.toggleMute()
                    timer.timeout.connect( resetVolumeChangedByUser );
                    timer.start(200);
                }
            }
        }
       
        Slider {
            id: sliderBar
            property bool isSeeking: false
            property int lastVolumeLevel: 0;
            
            leftMargin: 0
            rightMargin: 0
            
            verticalAlignment: VerticalAlignment.Center
            visible : volumeControlType == AudioDevice.Percentage

            //function to toggle the volume on/off while 
            //keeping the previous volume level 
            function toggleMute() {
                setVolumeChangedByUser();
                if ( value == 0 ) {
                    value = lastVolumeLevel > 0 ? lastVolumeLevel : 25;
                } else {
                    lastVolumeLevel = value;
                    value = 0;
                }
	   	        commonAnalytics.instance.logVolumeMuteToggled(value);
            }

            layoutProperties: StackLayoutProperties {	        
	            spaceQuota: 100
            }
        
            //range of volume levels
            fromValue: 0
            toValue: 100
        	
            onValueChanged: {
                setVolume(value);
                // if using headphones
                if (audioManager.activeAudioDevice.deviceType == AudioDevice.Headphone || audioManager.activeAudioDevice.deviceType == AudioDevice.Headset) {
	                // if safe zone is disabled but we've gone over the threshold, the audio is going to get capped to the threshold until the agreement is accepted
	                // in this case, we should set the slider back to what the system volume is
	                if (headphoneAudioDevice.unsafeZoneSupported && !headphoneAudioDevice.unsafeZoneEnabled && value > headphoneAudioDevice.unsafeZoneLevel) {
	                    sliderBar.value = audioManager.activeAudioDevice.outputLevel * 100;
	                }
	            }
            }

            onImmediateValueChanged: {
                setVolumeIcon(immediateValue);              
                if( isSeeking ){
                    saveVolume(immediateValue);
                    volumeControlContainer.volumeLevelChanging(immediateValue);
                }                
            }

            onTouch:{
                if (event.isDown()){
                    isSeeking = true;
                    setVolumeMode(true);
                    console.debug("seekingStateChanged in volumecontrol.qml: true ");
                    commonAnalytics.instance.logVolumeSliderPressed(value);
                }else if(event.isUp() || event.isCancel()){
                    isSeeking = false;
                    console.debug("seekingStateChanged in volumecontrol.qml: false");
                    setVolumeMode(false);
                    commonAnalytics.instance.logVolumeSliderReleased(value);
                }
	        }                        
        } //Slider
        ImageView {
            id: rightIcon
        
            verticalAlignment: VerticalAlignment.Center
            horizontalAlignment: HorizontalAlignment.Right
        
            //todo performance:  reuse this property set.. it came in on a merge
            layoutProperties: StackLayoutProperties {        	
            }
        
            onTouch: {
                if (event.isUp()) {
                    volumeControlContainer.rightImageTouchUp()
                }
            }
        }
    }//Percentage container
    //increase and decrease buttons    
    Container {
        
        leftPadding: 15
        rightPadding: 20 
        
        horizontalAlignment: HorizontalAlignment.Fill             
        verticalAlignment: VerticalAlignment.Fill
       
        topPadding: 16
        bottomPadding: 16
        layout: StackLayout {
            orientation: LayoutOrientation.LeftToRight
        }
        background: MediaApplication.volumeControlBackgroundColor         
        visible: volumeControlType == AudioDevice.Simple

        Label {
            id: label1
            textStyle
            {
                color: MediaApplication.textColor
                base: SystemDefaults.TextStyles.TitleText
                fontSize: FontSize.XLarge
            }
            text: qsTr("Volume") + Retranslate.onLanguageChanged
            verticalAlignment: VerticalAlignment.Center
            rightPadding: 44
            minWidth: 200
           
            horizontalAlignment: HorizontalAlignment.Fill
            layoutProperties: StackLayoutProperties {            
                spaceQuota: 100
            }
        }
        

        Button{
            id: decreaseButton
            objectName: "decreaseButton"

            //horizontalAlignment: HorizontalAlignment.Right
            verticalAlignment: VerticalAlignment.Center

            topPadding: 11
            bottomPadding: 11
            rightPadding: 1
            preferredWidth: 174
            onClicked: {
                volumeControlContainer.decreaseVolume();
            }
            layoutProperties: StackLayoutProperties {            
                spaceQuota: -1
            }
        }

        Button{
            id: increaseButton
            objectName: "increaseButton"
            
            //horizontalAlignment: HorizontalAlignment.Left
            verticalAlignment: VerticalAlignment.Center

            topPadding: 11
            bottomPadding: 11

            preferredWidth: 174
            onClicked: {
                volumeControlContainer.increaseVolume();
            }
            layoutProperties: StackLayoutProperties {            
                spaceQuota: -1
            }
        }

    }
}
