import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import bb.cascades.multimedia.internal 1.0
import "../core/layoutconstants.js" as LayoutConstants

/**
 * NowPlayingAlbumCover displays album art image on a container.
 */

Container {
    id: nowPlayingAlbumCover
    
    // Public properties
    property alias remoteRendererName: remoteRendererNameLabel.text
    
    property alias coverArtCutOpen: coverArtCutOpen
    property alias coverArtCutClosed: coverArtCutClosed
    
    property Container remoteInfo: remoteContainer 
    
    property int downPos // position of album cover when it is in open state
    property int downThreshold // the threshold when the scrolling will open the album cover
    property int upThreshold: downThreshold >> 2 // the threshold when the scrolling will close the album cover
    
    /** Should the eject button be shown on the screen? */
    property bool remoteInfoVisible: false
    
    /** Is this album cover for the full-screen mode? */
    property bool fullScreen: false    
    
    property bool stickyPlayOnMode: false
    
    property bool coverOpen: false
    
    // Private properties
    property bool __fingerDown
    property bool __scrolling: false
    property int __initY
    property int __initTranslationY
    
    property int albumCoverHeight: LayoutConstants.ALBUM_COVER_HEIGHT
    
    // Constants
    property string defaultAlbumArt: "asset:///images/common/Now_playing_default_album_art.png"

    // Signals
    signal coverOpenStarted();
    signal coverCloseStarted();
    signal scrollStarted();
    signal scrollProgress(int delta);
    signal scrollEnded(bool coverOpen);
    
    layout: DockLayout{}
    
    onTouch: {
        if (fullScreen) {
            return;
        }
        
        // If the eject button was also pressed, then ignore this event. It gets it first.
        if (ejectButton.ejectHandled) {
            ejectButton.ejectHandled = false;
        } else {
            if (event.isUp() && !__scrolling) {
                animateCover();
                return;
            }
        }
        
        if (event.isDown()){
            __fingerDown = true;
            __initY = event.localY;
            __initTranslationY = nowPlayingAlbumCover.translationY;
        } else if(event.isUp() || event.isCancel()){
            if (__scrolling) {
                __fingerDown  = false;
                __scrolling = false;
                var delta = nowPlayingAlbumCover.translationY - __initTranslationY;
                
                if (coverOpen) {
                    if (delta < 0 && -delta > upThreshold){
                        coverOpen = false;
                    }
                } else {
                    if (delta > 0 && delta > downThreshold){
                        coverOpen = true;
                    }
                }
                scrollEnded(coverOpen);
            }
        } else if(event.isMove() && __fingerDown){
            var delta = event.localY - __initY;
            
            if(delta > 20 || delta < -20){
                if (!__scrolling) {
	                scrollStarted();
	                __scrolling = true;
	            }

                delta = __initTranslationY + delta;                
                if(delta > 0){
                    nowPlayingAlbumCover.translationY = delta;
                    scrollProgress(delta);
                }
            }
        }
    }
    
    // function to open and close the cover
    function animateCover() {
        if (coverOpen) {
            coverCloseStarted();
        } else {
            coverOpenStarted();
        }
        coverOpen = !coverOpen;
    }
    
    function setAlbumArt(uri){
        previousAlbumArt.imageSource = albumArt.imageSource;
        if (uri != "" && uri != null) {
            albumArt.imageSource = uri;
        } else {
            albumArt.imageSource = defaultAlbumArt;
        }
    }
    
    Container {
        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Top
        
        layout: DockLayout {}
        background: ColorUtils.get(0xff121212)
        ImageView{
            id: previousAlbumArt
            preferredHeight: albumCoverHeight 
            minHeight: albumCoverHeight
            scalingMethod: ScalingMethod.Fill
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
        }
        
        ImageView {
            id: albumArt
            imageSource: defaultAlbumArt
            preferredHeight: albumCoverHeight
            minHeight: albumCoverHeight
            scalingMethod: ScalingMethod.Fill
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
        }
        
        ImageView {
            id: coverArtCutOpen
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
            visible: !fullScreen
            opacity: 0
            scalingMethod: ScalingMethod.Fill
            imageSource: "asset:///images/playing/full_album_shadow_open.png"
        }
        ImageView {
            id: coverArtCutClosed
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Top
            visible: !fullScreen
            opacity: 1
            scalingMethod: ScalingMethod.Fill
            imageSource: "asset:///images/playing/full_album_shadow.png"
        }
    }
        
    Container {
        layout: DockLayout {
        }

        horizontalAlignment: HorizontalAlignment.Fill
        verticalAlignment: VerticalAlignment.Bottom
        visible: remoteInfoVisible
        
        // This bottom padding is needed so that remoteContainer appears on the top of time slider.
        bottomPadding: LayoutConstants.REMOTE_INFO_BOTTOM_PADDING

        ControlDelegate {
            // Use ImageView for N-Series because the time slider on N-Series uses an ImageView.
            sourceComponent: LayoutConstants.TIME_SLIDER_USE_IMAGE_VIEW ? imageViewComponent : null
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Fill
        }

        Container {
            id: remoteContainer
    
            leftPadding: 40
            rightPadding: 40
            topPadding: 10
            bottomPadding: 10
            horizontalAlignment: HorizontalAlignment.Fill
            verticalAlignment: VerticalAlignment.Bottom

            background: LayoutConstants.REMOTE_INFO_COLOR
            opacity: LayoutConstants.REMOTE_INFO_OPACITY
                       
            Container {
                layout: StackLayout { orientation: LayoutOrientation.LeftToRight  }
                horizontalAlignment: HorizontalAlignment.Fill
                
                Container {
                    layout: StackLayout {}
    
                     layoutProperties: StackLayoutProperties {
                            spaceQuota: 100
                        }
                    
                    // the now playing on
                    Label {
                        id: remoteNowPlayingOnLabel
                        horizontalAlignment: HorizontalAlignment.Left
                        verticalAlignment: VerticalAlignment.Top
                        
                        //: text indicating we are playing on a remote renderer (labelled below)
                        text: (stickyPlayOnMode ? qsTr("Connected to: ") : qsTr("Playing on: ")) + Retranslate.onLanguageChanged
    
                        textStyle.base: SystemDefaults.TextStyles.SubtitleText
                        textStyle.fontSize: LayoutConstants.REMOTE_INFO_TOP_FONT_SIZE
                        textStyle.fontSizeValue: LayoutConstants.REMOTE_INFO_TOP_FONT_SIZE_VALUE
                        textStyle.color: ColorUtils.getPrimaryTextColor()
                        bottomMargin: 0
                    }
                
                    
                    // the dlna renderer name.
                    Label {
                        id: remoteRendererNameLabel
                        topMargin: 0
                        horizontalAlignment: HorizontalAlignment.Left
                        verticalAlignment: VerticalAlignment.Bottom
                        
                        textStyle.base: SystemDefaults.TextStyles.TitleText
                        textStyle.fontSize: LayoutConstants.REMOTE_INFO_LOWER_FONT_SIZE
                        textStyle.fontSizeValue: LayoutConstants.REMOTE_INFO_LOWER_FONT_SIZE_VALUE
                        textStyle.color: ColorUtils.getPrimaryTextColor()
                    }
                 }
                
                // The eject button
                MultiStateImageView {
                    id: ejectButton
                    
                    property bool ejectHandled: false
    
                     horizontalAlignment: HorizontalAlignment.Right
                     verticalAlignment: VerticalAlignment.Center
                    
                     upImages: ["asset:///images/playing/eject-icon.png"]
                     downImages: ["asset:///images/playing/eject_selected-icon.png"]
    
                     onTouch: {
                        ejectHandled = true;
                        if (event.isUp() && !__scrolling) {
                             ejectPressed();
                        }
                     }
                }
            }
        }
    }                   
}
