import bb.cascades 1.0
import bb.cascades.multimedia 1.0
import "../"
import "../core/layoutconstants.js" as LayoutConstants

/**
 * A label that will automatically scroll its contents back and forth if the content
 * exceeds the available space. (Marquee Effect)
 * 
 * Sample usage:
 * 
 * Container {
 *     id: myContainer
 *     
 *     horizontalAlignment: HorizontalAligment.Fill
 * 
 *     PanningLabel {
 *         id: myPanningLabel
 *         
 *         horizontalAlignment: HorizontalAligment.Fill
 *         
 *         labelText: "This is a long string of text to display"
 *     }
 *  
 * }
 * 
 */
Container {
	id: panningLabel

///*** <ExternalProperties>
    /**
     * Text to display in the label
     */
    property variant labelText: ""
    /**
     * Font size type to use for the label
     */
    property variant fontSize: FontSize.PointValue
    /**
     * Font size value to use
     */
    property variant fontSizeValue: 9.6
    /**
     * Color to set for the label
     */
    property variant color: Color.White
    /**
     * Font weight to use for the label
     */
    property variant fontWeight: FontWeight.W400
    /**
     * Top margin to use for the label
     */
    property variant labelTopMargin: 0
    
    property variant labelBottomMargin: 0
///*** </ExternalProperties>


///*** <InternalProperties>
	/**
	 * The amount of width needed for the label. This is an internal property only
     * and is set when the LayoutUpdateHandler for the first label is called.
	 */
	property int labelWidth: 0
    /**
     * The total width available to the container. This is an internal property only
     * and is set when the LayoutUpdateHandler for the outer visible container is called.
     */
    property int availableW: 0
    /**
     * The target translation for the marquee animation. This is an internal property only
     * that is calculated just before starting the animation.
     */
    property int animEndX: 0
    /**
     * Flag that tracks whether the labelText contains text that is to be displayed in
     * a right-to-left nature. This is an internal property only that is set/cleared
     * upon labelText changes.
     */
    property bool rightToLeft: false
///*** </InternalProperties>

///*** <InternalConstants>
	/**
	 * Timer that is needed since layoutFrameChanged is not guaranteed
	 */
	property int const_TIMER_ANIMATION_TIME:		2000
	/**
	 * Delay before the first animation starts
	 */
	property int const_ANIMATION_BEGIN_DELAY:		4000
    /**
     * Amount of time the animation takes
     */
    property int const_ANIMATION_DURATION:			15000
    /**
     * Distance between labels in pixels
     */
    property int const_SPACE_DISTANCE:				200
    /**
     * A 'semi-arbitrary' large value.
     * Use a maximum 24-bit value since it will
     * be used as a single precision floating point
     * parameter.
     */
    property int const_LARGE_VALUE:                 16777215 // ( 2^24 ) - 1
///*** </InternalConstants>

    // ScrollView is only needed to avoid clipping of the label it holds.  If it is just in the container
    // the the Label width will only be the container width and never more, even if it needs more.
    // The ScrollView allows it say the actual amount of total width it needs 
    ScrollView {
        id: mainScrollView
        scrollViewProperties.scrollMode: ScrollMode.Horizontal
        scrollViewProperties.pinchToZoomEnabled: false
        scrollViewProperties.overScrollEffectMode: OverScrollEffectMode.None
        horizontalAlignment: HorizontalAlignment.Fill

        // turn off touch so that you can't interact with the text
        touchPropagationMode: TouchPropagationMode.None

		Container {
		    id: outerContainer
		    horizontalAlignment: HorizontalAlignment.Fill

		    // stack the two labels beside each other to animate them in a rotating
		    // fashion
		    layout: StackLayout {
		        orientation: LayoutOrientation.LeftToRight
      		}
		
	        Label {
				id: label1
	            
	            text: panningLabel.labelText
				textStyle.fontSize: panningLabel.fontSize
		        textStyle.fontSizeValue: panningLabel.fontSizeValue
				textStyle.color: panningLabel.color
				textStyle.fontWeight: panningLabel.fontWeight
				topMargin: panningLabel.labelTopMargin
				bottomMargin: panningLabel.labelBottomMargin
				textFormat: TextFormat.Plain
                implicitLayoutAnimationsEnabled: false
                property int windowState: appContext.windowState

                onWindowStateChanged: {
                    // stop the marquee when the application is thumbnailed and restart
                    // when returned to fullscreen
                    if (windowState == 1) {
                        marqueeAnimation.stop();
                        timerAnimation.stop();
                    } else if (windowState == 0) {
                        timerAnimation.play();
                    }
                }

		        onTextChanged: {
		            // if the text changes, stop the animation which will reset the tranlsationX
		            // the layout update handler will be called and will then
		            // take care of starting the animation up again.
	                marqueeAnimation.stop();
                    timerAnimation.stop();
	                timerAnimation.play();
	                
                    // Hide the copy immediately, we will know later on
                    // if we want to show it again.
                    label2.opacity = 0.0;

                    rightToLeft = localeUtils.textIsRightToLeft( text );
                    if( rightToLeft ) {
                        // Change our container layout right to left
                        // and move the scroll view to the 'end'
                        outerContainer.layout.orientation = LayoutOrientation.RightToLeft;
                        mainScrollView.scrollToPoint( const_LARGE_VALUE, 0, ScrollAnimation.None );
                    } else {
                        // Change our container layout left to right
                        // and move the scroll view back to the 'beginning'
                        outerContainer.layout.orientation = LayoutOrientation.LeftToRight;
                        mainScrollView.scrollToPoint( 0, 0, ScrollAnimation.None );
                    }
		        }

	        	// Label's attached object
	        	attachedObjects: [
	                LayoutUpdateHandler {
	                    onLayoutFrameChanged: {
                            // Store the updated label width
                            labelWidth = layoutFrame.width;
	                    }
	                }
	            ] // attched object
	        } // Label1
	        
	        Label {
	            id: label2
	            
	            text: panningLabel.labelText
	            textStyle.fontSize: panningLabel.fontSize
	            textStyle.fontSizeValue: panningLabel.fontSizeValue
	            textStyle.color: panningLabel.color
	            textStyle.fontWeight: panningLabel.fontWeight
	            topMargin: panningLabel.labelTopMargin
	            bottomMargin: panningLabel.labelBottomMargin
	            textFormat: TextFormat.Plain
	            leftMargin: const_SPACE_DISTANCE
	            rightMargin: const_SPACE_DISTANCE
	            
	            // set the opacity to 0 initially.  only make this second label visible
	            // if the text is long enough to require the two labels and animation
	            opacity: 0.0

	            // Disable the fade animation too (invoked by changing opacity)
                attachedObjects: [
                    ImplicitAnimationController {
                        propertyName: "opacity"
                        enabled: false
                    }
                ] // attached object
	        } // Label2
	        
	        animations: [
	            TranslateTransition {
	                id: timerAnimation
	                
	                fromX: 0
	                toX: 0
	                duration: const_TIMER_ANIMATION_TIME
	                
	                onEnded: {
                        // if the label is too wide, then we want to start the animation
                        // and we also want to set the second label to be visible
                        if (labelWidth > availableW) {
                            // Do our final animation calculation ...
                            var magnitude = labelWidth + const_SPACE_DISTANCE;
                            if( !rightToLeft ) {
                                // We will be scrolling in the negative
                                // direction for non right-to-left text
                                magnitude *= -1;
                            }
                            // ... and set it
                            animEndX = magnitude;

                            // Show our label copy and kick off the animation
                            label2.opacity = 1.0;
                            marqueeAnimation.play();
                        } else {
                            label2.opacity = 0.0;
                        }
                    }
	            },
	            // Main marquee animation
	            SequentialAnimation {
	                id: marqueeAnimation
	                
	                repeatCount: AnimationRepeatCount.Forever
	                
	                TranslateTransition {
	                    // this transition is somewhat of a hack.  its purpose is to delay the start of
	                    // the actual marquee animation.  it can be replaced by the "delay" property
	                    // on the animation itself, however, for whatever reason, the delay property
	                    // is not working currently
	                    easingCurve: StockCurve.Linear
	                    fromX: 0
	                    toX: 0
	                    duration: const_ANIMATION_BEGIN_DELAY
	                }
	        		TranslateTransition {
			            // translate the container from the starting position of the first label
			            // to the x coordinate of the second label.  this brings the second label
			            // into the same position that the first label was in
			            easingCurve: StockCurve.SineInOut
			            fromX: 0
			            toX: animEndX
			            duration: const_ANIMATION_DURATION
	              	}
	
		        	onStopped: {
		                // when the animation is stopped, reset the container
		                // to it's original position
		                outerContainer.translationX = 0;
		        	}
	            }
	        ] // animations
	        
	        // inner container's attached object
	        attachedObjects: [
	            // turn off translation implicit animation so that as the container position
	            // is reset, there is no animation that happens
			    ImplicitAnimationController {
		            id: translationController
		            propertyName: "translationX"
		            enabled: false
			    }
	        ] // attached object
	
		} // Container

    } // ScrollView

    // Main container's attached object
    attachedObjects: [
	    LayoutUpdateHandler {
		    onLayoutFrameChanged: {
			    // Get the width of the visible outer container
			    availableW = layoutFrame.width;
		    }
	    },
        MediaLocaleUtils {
            id: localeUtils
        }
    ] // attached object
    
} // Container
