import bb.cascades 1.2
import bb.cascades.multimedia 1.0
import bb.multimedia 1.0
import bb.system 1.2
import bb.casades.music 1.0

Container {
    property bool fmRadioSupported: true
    property variant favouritesDataModel: favourites
    property variant stationsDataModel: stations
    property variant playlistController: controller
    property variant radioPlayer: player
    property variant radioScanner: scanner
    property variant nowPlaying: nowPlayingService
    property variant volumeController: radioVolumeController

    property string currentRadioFrequency
    property bool 	isCurrentFrequencyFavourite
        
    property bool headphonesConnected: headsetAudioDevice.connected || headphonesAudioDevice.connected
    property bool airplaneModeOn: (networkStatusManager.airplaneMode == NetworkStatusManager.AM_On)  
    property bool radioPlaybackActive
    
    // Keeps track if the Radio player screen has been closed at some point
    // during the app lifetime
    property bool radioPlayerHasBeenClosed:false
    
    signal radioPlayerClosed()
    signal headphoneConnectionStatusChanged(bool connected);
    signal airplaneModeEnabled()

    attachedObjects: [
        RadioPlaylistController{
            id: controller
        },
        RadioPlayer {
            id:player
        },        
        VolumeController{
            id: radioVolumeController
        },
        NormalizedFrequencyDecorator {
            id: favourites
            frequencyKey: "frequency"
            normalizedKey: "title"
            frequencyStep: player.frequencyParams.stepping
            model: PersistableDataModel {
                storagePath: "data/fm.db"
                tableName: "favourites"
                onItemAdded:{
                    if (radioPlaybackActive) {
                        isCurrentFrequencyFavourite = favourites.model.contains(radioPlayer.frequency);
                    }
                }
                onItemRemoved:{
                    if (radioPlaybackActive) {
                        isCurrentFrequencyFavourite = favourites.model.contains(radioPlayer.frequency);
                    }
                }
            }
        },
        PropertyProvider {
            id: favouritesProvider
            propertyName: "isfavourite"
            provider: favourites.model
        },
        NormalizedFrequencyDecorator {
            id: stations
            frequencyKey: "frequency"
            normalizedKey: "title"
            frequencyStep: player.frequencyParams.stepping
            model: ScannedStationsDataModel {
                propertyProvider: favouritesProvider
                storagePath: "data/fm.db"
                tableName: "stations"
            }
        },
        HeadsetAudioDevice {
            id: headsetAudioDevice
            onConnectedChanged: {
                handleHeadphoneAudioDeviceConnectionChanged(connected);
            }
        },
        HeadphoneAudioDevice {
            id: headphonesAudioDevice
            onConnectedChanged: {
                handleHeadphoneAudioDeviceConnectionChanged(connected);
            }
        }, 
        NowPlayingConnection {
            id: nowPlayingService
            nextEnabled:playlistController.nextAvailable
            previousEnabled:playlistController.prevAvailable
            
            onRevoked: {
                console.log("Radio revoked by: " + nowPlayingService.lastRevokingConnectionPid + " , our pid: " + appPid);
                if (radioPlaybackActive) {
	                if (appPid == nowPlayingService.lastRevokingConnectionPid) {
	                    stopPlayback();
	                } else {
	                    radioPlayer.pause();
	                }
	            }
            }
        },
        Scanner {
            id: scanner           
        }, 
        NetworkStatusManager {
            id: networkStatusManager
            
            onAirplaneModeChanged: {
                //If airplane mode is on, or about to go on, then stop playback if necessary
                if (airplaneMode == NetworkStatusManager.AM_Pending_On || airplaneMode == NetworkStatusManager.AM_On) {
                    airplaneModeEnabled();
                    if (radioPlaybackActive) {
                    	stopPlayback();
                    	console.log("Stopped Radio Playback due to Airplane Mode");
                    }
                }
            }
        }
    ]
    
    onCreationCompleted: {
        console.debug("+RadioPlaybackContext.onCreationCompleted:")
        // Signal to update the current frequency string
        radioPlayer.frequencyChanged.connect(handleCurrentFrequencyChanged);

		// Connect signals between the playlist controller and the player
        playlistController.currentFrequencyChanged.connect(setFrequency);
        radioPlayer.frequencyChanged.connect(playlistController.setCurrentFrequency);
        
        radioPlayer.mediaStateChanged.connect(onMediaStateChange);

        radioPlayer.mediaStateChanged.connect(nowPlaying.setMediaState);
        radioPlayer.metaDataChanged.connect(nowPlaying.setMetaData);
        
        radioPlayer.seekCompleted.connect(stations.model.onSeekCompleted);
        radioPlayer.scanStarted.connect(stations.model.onScanStarted);
        radioPlayer.scanCompleted.connect(stations.model.onScanCompleted);
        radioPlayer.scanStopped.connect(stations.model.onScanStopped);
        
        radioPlayer.frequencyFound.connect(stations.model.onFrequencyFound);
        
        // Set up Now Playing
        nowPlaying.pause.connect(radioPlayer.pause);
        nowPlaying.play.connect(radioPlayer.play);
        nowPlaying.stop.connect(radioPlayer.stop);
        nowPlaying.next.connect(playlistController.next);
        nowPlaying.previous.connect(playlistController.prev);
        
        nowPlaying.setIconUrl("asset:///images/radio/fmradio_default_album_big.png");

        stations.model.load();
        initModel(favourites.model);
    }
          
    function cleanup() {
        console.debug("+ RadioPlaybackContext.cleanup");
        // Signal to update the current frequency string
        radioPlayer.frequencyChanged.disconnect(handleCurrentFrequencyChanged);
        
        // Connect signals between the playlist controller and the player
        playlistController.currentFrequencyChanged.disconnect(setFrequency);
        radioPlayer.frequencyChanged.disconnect(playlistController.setCurrentFrequency);
        
        console.debug("- RadioPlaybackContext.cleanup");
    }
    
    function onMediaStateChange(mediaState){
        console.debug("+RadioPlaybackContext.onMediaStateChange() " + mediaState);	
        
        radioPlaybackActive = mediaState == MediaState.Started || mediaState == MediaState.Paused
        
        switch(mediaState){
            case MediaState.Unprepared:
            case MediaState.Stopped:
                //Clear our state
                currentRadioFrequency = ""   
                isCurrentFrequencyFavourite = false;
                
                playlistController.playlist = null;
                
                if (nowPlaying.acquired) {
                    nowPlaying.revoke();
                }            
                break;
        }
        console.debug("-RadioPlaybackContext.onMediaStateChange()");
    }

    function addFavourite(frequency){
        console.log("adding favourite: " + frequency)
        var f = {
            frequency: frequency,
            station_name: ""
        };
        favourites.model.insert(f);
        favourites.model.save();
    }
    
    function removeFavourite(frequency){
        console.log("removing favourite: " + frequency)
        var f = {
            frequency: frequency,
            station_name: ""
        };
        
        var index = favourites.model.find(f);        
        favourites.model.removeAt(index);
        favourites.model.save();
    }
        
    function setFrequency(frequency) {
        console.debug(" + RadioPlaybackContext::setFrequency()");

        if (frequency != 0) {
	        if (!nowPlaying.acquired) {
	            console.debug("NPC not acquired");
	            
	            //Try to acquire
	            var error = nowPlaying.acquire();
	            if(error == MediaError.None){
	                radioPlayer.frequency = frequency;
	            } else {
	                console.debug("Failed to acquire NPC");
	            }
	            	
	        } else {
	            console.debug("NPC is acquired");
	            radioPlayer.frequency = frequency;
	        }
	    }
        console.debug(" - RadioPlaybackContext::setFrequency()");
    }
    
    function handleHeadphoneAudioDeviceConnectionChanged(connected) {
        console.debug("Headset connected " + connected);
        
        // If headset was disconnected and we are playing, stop playback and show a toast
        if (!connected && radioPlaybackActive) {
            stopPlayback();
            systemDialogUtils.showSystemToast(qsTr("Headset is removed, please plug in the headset to continue listening to FM Radio."));
        }
        
        if(!RadioPlaybackContext.stationsDataModel.hasChildren([])) {
            //If Local stations list is empty, kick off a scan to find some stations
            scanner.scan();
        }
        
        headphoneConnectionStatusChanged(connected);
    }
    
    function handleCurrentFrequencyChanged (frequency) {
        isCurrentFrequencyFavourite = favourites.model.contains(frequency);
                
        // Convert from Hz to MHz to display the proper string
        currentRadioFrequency = formatFrequency(frequency);
        var metaData = {"title" : currentRadioFrequency};
        nowPlaying.setMetaData(metaData);
        console.log("New frequency is " + currentRadioFrequency);
    }    
    
    function formatFrequency(frequency) {
        var newFrequency = frequency / 1000000;
        return newFrequency.toString();
    }
    
    function onRadioPlayerScreenPopped() {
        console.debug("Radio player screen popped..");
        radioPlayerHasBeenClosed = true;
        radioPlayerClosed();
    }  
    
    function stopPlayback(){
        console.debug("+RadioPlaybackContext stopPlayback " + radioPlaybackActive);
        if (radioPlaybackActive){
            playlistController.playlist = null;
            radioPlayer.reset();
        }
        
        if (nowPlaying.acquired) {
            nowPlaying.revoke();
        }
        console.debug("-RadioPlaybackContext stopPlayback");
    }
    
    // Starts radio playback at the given index in the data model and launches the player screen
    function playIndex (dataModel,indexPath) {
        console.log("+ playIndex")

        // Stop DLNA playback before we start radio playback
        if (appContext.playbackController == remotePlaybackController) {
        	appContext.switchToLocalPlayback();
        }

        // Only push the player screen if we are not currently playing
        playlistController.playlist = dataModel
        
        if (!radioPlaybackActive){
            console.log("pushing RadioPlayerScreen");
            tabbedPane.currentNavPane.safePushOnePage("/radio/player/RadioPlayerScreen.qml");
        }
                
        playlistController.currentIndex = indexPath
             	
        console.log("- playIndex")
    }
    
    // Launches the Radio Player Screen if radio playback is active
    function launchRadioPlayerScreen () {
        console.log("+ launchRadioPlayerScreen")
        
        if (radioPlaybackActive){
            tabbedPane.currentNavPane.safePushOnePage("/radio/player/RadioPlayerScreen.qml");
        }
        
        console.log("- launchRadioPlayerScreen")
    }
    
    // Launches the Radio Player Screen if radio playback is active
    function launchPlayerScreenForTuneIn() {
        console.log("+ launchPlayerScreenForTuneIn")
        
        if (playlistController.playlist == null){
            playlistController.playlist = stationsDataModel;
        }
        
        var playerScreen = tabbedPane.currentNavPane.safePushOnePage("/radio/player/RadioPlayerScreen.qml");
        
        playerScreen.requestTuneBoxFocus();
        
        console.log("- launchPlayerScreenForTuneIn")
    }
    
    function tuneToStation(station) {
        console.log("+ tuneToStation")
                
        // Convert from MHz to Hz.
        setFrequency(station * 1000000);
        
        console.log("- tuneToStation")
    }
    
    
    /*----------------------TEST VARIABLES/METHODS START----------------------
     * TODO: The following variables and methods are for testing purposes
     * They need to be removed as we integrate properly with RadioPlayer
     */    
    property bool showEmptyFavoritesList: false
    property bool showEmptyStationsList: false 

    function initModel(model) {
        // Hacky way to pull data out of the db on start up.
        var schema = {
            frequency: PersistableDataType.Integer,
            station_name: PersistableDataType.Text
        };
        model.setSchema(schema, "frequency");
        model.load();
    }
}

